import { Runtime, Integration, State, Storage, StorageConfigInterface, Collection, DefaultItem, Computed, Event, EventConfig, DefaultEventPayload, Integrations, Observer, SubController } from "./internal";
export declare class Agile {
    config: AgileConfigInterface;
    runtime: Runtime;
    subController: SubController;
    storage: Storage;
    integrations: Integrations;
    static initialIntegrations: Integration[];
    /**
     * @public
     * Agile - Global state and logic framework for reactive Typescript & Javascript applications
     * @param config - Config
     */
    constructor(config?: AgileConfigInterface);
    /**
     * @public
     * Integrates framework into Agile
     * @param integration - Integration that gets registered/integrated
     */
    use(integration: Integration): this;
    /**
     * @public
     * Storage - Handy Interface for storing Items permanently
     * @param config - Config
     */
    Storage: (config: StorageConfigInterface) => Storage;
    /**
     * @public
     * State - Class that holds one Value and causes rerender on subscribed Components
     * @param initialValue - Initial Value of the State
     * @param key - Key/Name of the State
     */
    State: <ValueType>(initialValue: ValueType, key?: string | undefined) => State<ValueType>;
    /**
     * @public
     * Collection - Class that holds a List of Objects with key and causes rerender on subscribed Components
     * @param config - Config
     */
    Collection: <DataType = DefaultItem>(config?: import("./collection").CollectionConfigInterface | ((collection: Collection<DataType>) => import("./collection").CollectionConfigInterface) | undefined) => Collection<DataType>;
    /**
     * @public
     * Computed - Function that recomputes its value if a dependency changes
     * @param computeFunction - Function for computing value
     * @param deps - Hard coded dependencies of Computed Function
     */
    Computed: <ComputedValueType = any>(computeFunction: () => ComputedValueType, deps?: (Observer<any> | State<any> | Event<DefaultEventPayload>)[] | undefined) => Computed<ComputedValueType>;
    /**
     * @public
     * Event - Class that holds a List of Functions which can be triggered at the same time
     * @param config - Config
     */
    Event: <PayloadType = DefaultEventPayload>(config?: EventConfig | undefined) => Event<PayloadType>;
    /**
     * @public
     * Configures Agile Storage
     * @param storage - Storage that will get used as Agile Storage
     */
    configureStorage(storage: Storage): void;
    /**
     * @public
     * Checks if Agile has any registered Integration
     */
    hasIntegration(): boolean;
}
/**
 * @param logJobs - Allow Agile Logs
 * @param waitForMount - If Agile should wait until the component mounts
 * @param storageConfig - To configure Agile Storage
 */
export interface AgileConfigInterface {
    logJobs?: boolean;
    waitForMount?: boolean;
    storageConfig?: StorageConfigInterface;
}
