"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Agile = void 0;
const internal_1 = require("./internal");
class Agile {
    /**
     * @public
     * Agile - Global state and logic framework for reactive Typescript & Javascript applications
     * @param config - Config
     */
    constructor(config = {}) {
        this.config = config;
        //=========================================================================================================
        // Storage
        //=========================================================================================================
        /**
         * @public
         * Storage - Handy Interface for storing Items permanently
         * @param config - Config
         */
        this.Storage = (config) => new internal_1.Storage(config);
        //=========================================================================================================
        // State
        //=========================================================================================================
        /**
         * @public
         * State - Class that holds one Value and causes rerender on subscribed Components
         * @param initialValue - Initial Value of the State
         * @param key - Key/Name of the State
         */
        this.State = (initialValue, key) => new internal_1.State(this, initialValue, key);
        //=========================================================================================================
        // Collection
        //=========================================================================================================
        /**
         * @public
         * Collection - Class that holds a List of Objects with key and causes rerender on subscribed Components
         * @param config - Config
         */
        this.Collection = (config) => new internal_1.Collection(this, config);
        //=========================================================================================================
        // Computed
        //=========================================================================================================
        /**
         * @public
         * Computed - Function that recomputes its value if a dependency changes
         * @param computeFunction - Function for computing value
         * @param deps - Hard coded dependencies of Computed Function
         */
        this.Computed = (computeFunction, deps
        // @ts-ignore
        ) => new internal_1.Computed(this, computeFunction, deps);
        //=========================================================================================================
        // Event
        //=========================================================================================================
        /**
         * @public
         * Event - Class that holds a List of Functions which can be triggered at the same time
         * @param config - Config
         */
        this.Event = (config) => new internal_1.Event(this, config);
        this.integrations = new internal_1.Integrations(this);
        this.runtime = new internal_1.Runtime(this);
        this.subController = new internal_1.SubController(this);
        this.storage = new internal_1.Storage(config.storageConfig || {});
        // Create global instance of Agile
        internal_1.globalBind("__agile__", this);
    }
    //=========================================================================================================
    // Use
    //=========================================================================================================
    /**
     * @public
     * Integrates framework into Agile
     * @param integration - Integration that gets registered/integrated
     */
    use(integration) {
        this.integrations.integrate(integration);
        return this;
    }
    //=========================================================================================================
    // Set Storage
    //=========================================================================================================
    /**
     * @public
     * Configures Agile Storage
     * @param storage - Storage that will get used as Agile Storage
     */
    configureStorage(storage) {
        // Get Observers that are already saved into a storage
        const persistentInstances = this.storage.persistentInstances;
        // Define new Storage
        this.storage = storage;
        // Transfer already saved items into new Storage
        persistentInstances.forEach((persistent) => persistent.initialLoading(persistent.key));
    }
    //=========================================================================================================
    // Has Integration
    //=========================================================================================================
    /**
     * @public
     * Checks if Agile has any registered Integration
     */
    hasIntegration() {
        return this.integrations.hasIntegration();
    }
}
exports.Agile = Agile;
Agile.initialIntegrations = []; // External added Integrations
