"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CollectionPersistent = void 0;
const internal_1 = require("../internal");
class CollectionPersistent extends internal_1.Persistent {
    /**
     * @internal
     * Collection Persist Manager - Handles permanent storing of Collection Value
     * @param agileInstance - An instance of Agile
     * @param collection - Collection that gets stored
     * @param key - Key of Storage property
     */
    constructor(agileInstance, collection, key) {
        super(agileInstance);
        this.defaultGroupSideEffectKey = "rebuildStorage";
        this.collection = () => collection;
        this.instantiatePersistent(key).then((success) => {
            collection.isPersisted = success;
        });
    }
    set key(value) {
        this.setKey(value);
    }
    get key() {
        return this._key;
    }
    //=========================================================================================================
    // Set Key
    //=========================================================================================================
    /**
     * @public
     * Sets Key/Name of Persistent
     * @param value - New Key/Name of Persistent
     */
    setKey(value) {
        return __awaiter(this, void 0, void 0, function* () {
            // If persistent isn't ready try to init it with the new Key
            if (!this.ready) {
                this.instantiatePersistent(value).then((success) => {
                    this.collection().isPersisted = success;
                });
                return;
            }
            // Check if key has changed
            if (value === this._key)
                return;
            // Remove value with old Key
            yield this.removeValue();
            // Update Key
            this._key = value;
            // Set value with new Key
            yield this.updateValue();
        });
    }
    //=========================================================================================================
    // Load Value
    //=========================================================================================================
    /**
     * @internal
     * Loads Value from Storage
     * @return Success?
     */
    loadValue() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.ready)
                return false;
            // Check if Collection is Persisted
            const isPersisted = yield this.agileInstance().storage.get(this.key);
            if (!isPersisted)
                return false;
            // Load Values into Collection
            const loadValuesIntoCollection = () => __awaiter(this, void 0, void 0, function* () {
                var _a, _b;
                const primaryKey = this.collection().config.primaryKey || "id";
                // Get Default Group
                const defaultGroup = this.collection().getGroup(this.collection().config.defaultGroupKey || "default");
                // Persist Default Group and instantiate it manually to await its instantiation
                const groupStorageKey = CollectionPersistent.getGroupStorageKey(defaultGroup.key, this.collection().key);
                defaultGroup.persist(groupStorageKey, { instantiate: false });
                defaultGroup.isPersisted =
                    (yield ((_a = defaultGroup.persistent) === null || _a === void 0 ? void 0 : _a.instantiatePersistent(groupStorageKey))) || false;
                // Add sideEffect to default Group which adds and removes Items from the Storage depending on the Group Value
                if (!defaultGroup.hasSideEffect(this.defaultGroupSideEffectKey))
                    defaultGroup.addSideEffect(this.defaultGroupSideEffectKey, () => this.rebuildStorageSideEffect(defaultGroup));
                // Load Storage Value from Items
                for (let itemKey of defaultGroup.value) {
                    // Get Storage Value
                    const storageValue = yield this.agileInstance().storage.get(CollectionPersistent.getItemStorageKey(itemKey, this.collection().key));
                    if (!storageValue)
                        continue;
                    // Collect found Storage Value
                    this.collection().collect(storageValue);
                    // Persist found Item that got created out of the Storage Value
                    (_b = this.collection()
                        .getItemById(storageValue[primaryKey])) === null || _b === void 0 ? void 0 : _b.persist(CollectionPersistent.getItemStorageKey(itemKey, this.collection().key));
                }
            });
            yield loadValuesIntoCollection();
            return true;
        });
    }
    //=========================================================================================================
    // Set Value
    //=========================================================================================================
    /**
     * @internal
     * Saves/Updates Value in Storage
     * @return Success?
     */
    updateValue() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.ready)
                return false;
            // Set Collection to Persisted (in Storage)
            this.agileInstance().storage.set(this.key, true);
            // Get default Group
            const defaultGroup = this.collection().getGroup(this.collection().config.defaultGroupKey || "default");
            // Persist default Group
            defaultGroup.persist({ followCollectionPattern: true });
            // Add sideEffect to default Group which adds and removes Items from the Storage depending on the Group Value
            if (!defaultGroup.hasSideEffect(this.defaultGroupSideEffectKey))
                defaultGroup.addSideEffect(this.defaultGroupSideEffectKey, () => this.rebuildStorageSideEffect(defaultGroup));
            // Persist Collection Items
            for (let itemKey of defaultGroup.value) {
                const item = this.collection().getItemById(itemKey);
                const itemStorageKey = CollectionPersistent.getItemStorageKey(itemKey, this.collection().key);
                item === null || item === void 0 ? void 0 : item.persist(itemStorageKey);
            }
            this.collection().isPersisted = true;
            return true;
        });
    }
    //=========================================================================================================
    // Remove Value
    //=========================================================================================================
    /**
     * @internal
     * Removes Value form Storage
     * @return Success?
     */
    removeValue() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.ready)
                return false;
            // Set Collection to not Persisted
            this.agileInstance().storage.remove(this.key);
            // Get default Group
            const defaultGroup = this.collection().getGroup(this.collection().config.defaultGroupKey || "default");
            // Remove default Group from Storage
            (_a = defaultGroup.persistent) === null || _a === void 0 ? void 0 : _a.removeValue();
            // Remove sideEffect from default Group
            defaultGroup.removeSideEffect(this.defaultGroupSideEffectKey);
            // Remove Collection Items from Storage
            for (let itemKey of defaultGroup.value) {
                const item = this.collection().getItemById(itemKey);
                (_b = item === null || item === void 0 ? void 0 : item.persistent) === null || _b === void 0 ? void 0 : _b.removeValue();
            }
            this.collection().isPersisted = false;
            return false;
        });
    }
    //=========================================================================================================
    // Validate Key
    //=========================================================================================================
    /**
     * @internal
     * Validates Storage Key
     * @param key - Key that gets validated
     */
    validateKey(key) {
        const collection = this.collection();
        // Get key from Collection
        if (!key && collection.key)
            return collection.key;
        // Return null if no key found
        if (!key)
            return null;
        // Set Storage Key to Collection Key if Collection has no key
        if (!collection.key)
            collection.key = key;
        return key;
    }
    //=========================================================================================================
    // Rebuild Storage SideEffect
    //=========================================================================================================
    /**
     * @internal
     * Rebuilds Storage depending on Group
     * @param group - Group
     */
    rebuildStorageSideEffect(group) {
        const collection = group.collection();
        // Return if only an ItemKey got updated -> length stayed the same
        if (group.previousStateValue.length === group.value.length)
            return;
        const addedKeys = group.value.filter((key) => !group.previousStateValue.includes(key));
        const removedKeys = group.previousStateValue.filter((key) => !group.value.includes(key));
        // Persist Added Keys
        addedKeys.forEach((itemKey) => {
            const item = collection.getItemById(itemKey);
            if (!(item === null || item === void 0 ? void 0 : item.isPersisted))
                item === null || item === void 0 ? void 0 : item.persist(CollectionPersistent.getItemStorageKey(itemKey, collection.key));
        });
        // Unpersist removed Keys
        removedKeys.forEach((itemKey) => {
            var _a;
            const item = collection.getItemById(itemKey);
            if (item === null || item === void 0 ? void 0 : item.isPersisted)
                (_a = item === null || item === void 0 ? void 0 : item.persistent) === null || _a === void 0 ? void 0 : _a.removeValue();
        });
    }
    //=========================================================================================================
    // Get Item Storage Key
    //=========================================================================================================
    /**
     * @internal
     * Build Item StorageKey with Collection Persist Pattern
     * @param itemKey - Key of Item
     * @param collectionKey - Key of Collection
     */
    static getItemStorageKey(itemKey, collectionKey) {
        if (!itemKey) {
            console.error("Agile: Failed to build Item StorageKey");
            itemKey = "unknown";
        }
        if (!collectionKey) {
            console.error("Agile: Failed to build Item StorageKey");
            collectionKey = "unknown";
        }
        return this.storageItemKeyPattern
            .replace("${collectionKey}", collectionKey.toString())
            .replace("${itemKey}", itemKey.toString());
    }
    //=========================================================================================================
    // Get Group Storage Key
    //=========================================================================================================
    /**
     * @internal
     * Build Group StorageKey with Collection Persist Pattern
     * @param groupKey - Key of Group
     * @param collectionKey - Key of Collection
     */
    static getGroupStorageKey(groupKey, collectionKey) {
        if (!groupKey) {
            console.error("Agile: Failed to build Group StorageKey");
            groupKey = "unknown";
        }
        if (!collectionKey) {
            console.error("Agile: Failed to build Group StorageKey");
            collectionKey = "unknown";
        }
        return this.storageGroupKeyPattern
            .replace("${collectionKey}", collectionKey.toString())
            .replace("${groupKey}", groupKey.toString());
    }
}
exports.CollectionPersistent = CollectionPersistent;
CollectionPersistent.storageItemKeyPattern = "_${collectionKey}_item_${itemKey}";
CollectionPersistent.storageGroupKeyPattern = "_${collectionKey}_group_${groupKey}";
