"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Group = void 0;
const internal_1 = require("../internal");
class Group extends internal_1.State {
    /**
     * @public
     * Group - Holds Items of Collection
     * @param agileInstance - An instance of Agile
     * @param collection - Collection to that the Group belongs
     * @param initialItems - Initial Key of Items in this Group
     * @param config - Config
     */
    constructor(agileInstance, collection, initialItems, config) {
        super(agileInstance, initialItems || [], config === null || config === void 0 ? void 0 : config.key);
        this._output = []; // Output of Group
        this._items = []; // Items of Group
        this.notFoundItemKeys = []; // Contains all keys of Group that can't be found in Collection
        this.collection = () => collection;
        // Add rebuild to sideEffects so that it rebuilds the Group Output if the value changes
        this.addSideEffect("buildGroup", () => this.rebuild());
        // Initial Build
        this.rebuild();
    }
    /**
     * @public
     * Get Item Values of Group
     */
    get output() {
        // Add Group to tracked Observers (for auto tracking used observers in computed function)
        if (this.agileInstance().runtime.trackObservers)
            this.agileInstance().runtime.foundObservers.add(this.observer);
        return this._output;
    }
    /**
     * @public
     * Get Items of Group
     */
    get items() {
        // Add Group to tracked Observers (for auto tracking used observers in computed function)
        if (this.agileInstance().runtime.trackObservers)
            this.agileInstance().runtime.foundObservers.add(this.observer);
        return this._items.map((item) => item());
    }
    //=========================================================================================================
    // Has
    //=========================================================================================================
    /**
     * @public
     * Checks if Group contains ItemKey
     * @param itemKey - ItemKey that gets checked
     */
    has(itemKey) {
        return this.value.findIndex((key) => key === itemKey) !== -1;
    }
    //=========================================================================================================
    // Size
    //=========================================================================================================
    /**
     * @public
     * Get size of Group (-> How many Items it contains)
     */
    get size() {
        return this.value.length;
    }
    //=========================================================================================================
    // Remove
    //=========================================================================================================
    /**
     * @public
     * Removes ItemKey/s from Group
     * @param itemKeys - ItemKey/s that get removed from Group
     * @param config - Config
     */
    remove(itemKeys, config = {}) {
        const _itemKeys = internal_1.normalizeArray(itemKeys);
        const notExistingItemKeys = [];
        let newGroupValue = internal_1.copy(this.nextStateValue); // Copying nextStateValue because somehow a reference exists between nextStateValue and value
        config = internal_1.defineConfig(config, {
            background: false,
        });
        // Remove ItemKeys from Group
        _itemKeys.forEach((itemKey) => {
            // Check if itemKey exists in Group
            if (!newGroupValue.includes(itemKey)) {
                console.error(`Agile: Couldn't find itemKey '${itemKey}' in Group!`, this);
                return;
            }
            // Check if ItemKey exists in Collection
            if (!this.collection().getItemById(itemKey))
                notExistingItemKeys.push(itemKey);
            // Remove ItemKey from Group
            newGroupValue = newGroupValue.filter((key) => key !== itemKey);
        });
        this.nextStateValue = newGroupValue;
        // If all removed ItemKeys doesn't exist in Collection -> no rerender necessary since output doesn't change
        if (notExistingItemKeys.length >= _itemKeys.length)
            config.background = true;
        // Ingest nextStateValue into Runtime
        this.ingest({ background: config.background });
        return this;
    }
    //=========================================================================================================
    // Add
    //=========================================================================================================
    /**
     * @public
     * Adds ItemKey/s to Group
     * @param itemKeys - ItemKey/s that get added to the Group
     * @param config - Config
     */
    add(itemKeys, config = {}) {
        const _itemKeys = internal_1.normalizeArray(itemKeys);
        const notExistingItemKeys = []; // ItemKeys that don't exist in Collection
        let newGroupValue = internal_1.copy(this.nextStateValue); // Copying nextStateValue because somehow a reference exists between nextStateValue and value
        config = internal_1.defineConfig(config, {
            method: "push",
            overwrite: false,
            background: false,
        });
        // Add ItemKeys to Group
        _itemKeys.forEach((itemKey) => {
            const existsInGroup = newGroupValue.includes(itemKey);
            // Check if ItemKey exists in Collection
            if (!this.collection().getItemById(itemKey))
                notExistingItemKeys.push(itemKey);
            // Remove ItemKey from Group if it should get overwritten and exists
            if (existsInGroup) {
                if (config.overwrite)
                    newGroupValue = newGroupValue.filter((key) => key !== itemKey);
                else
                    return;
            }
            // Add new ItemKey to Group
            newGroupValue[config.method || "push"](itemKey);
        });
        this.nextStateValue = newGroupValue;
        // If all added ItemKeys doesn't exist in Collection -> no rerender necessary since output doesn't change
        if (notExistingItemKeys.length >= _itemKeys.length)
            config.background = true;
        // Ingest nextStateValue into Runtime
        this.ingest({ background: config.background });
        return this;
    }
    persist(keyOrConfig = {}, config = {}) {
        let _config;
        let key;
        if (internal_1.isValidObject(keyOrConfig)) {
            _config = keyOrConfig;
            key = undefined;
        }
        else {
            _config = config || {};
            key = keyOrConfig;
        }
        _config = internal_1.defineConfig(_config, {
            instantiate: true,
            followCollectionPattern: false,
        });
        if (_config.followCollectionPattern) {
            key = internal_1.CollectionPersistent.getGroupStorageKey(key || this.key, this.collection().key);
        }
        super.persist(key, { instantiate: _config.instantiate });
        return this;
    }
    //=========================================================================================================
    // Rebuild
    //=========================================================================================================
    /**
     * @internal
     * Rebuilds Output and Items of Group
     */
    rebuild() {
        const notFoundItemKeys = []; // Item Keys that couldn't be found in Collection
        const groupItems = [];
        let groupOutput;
        // Create groupItems by finding Item at ItemKey in Collection
        this._value.forEach((itemKey) => {
            let data = this.collection().data[itemKey];
            if (data)
                groupItems.push(data);
            else
                notFoundItemKeys.push(itemKey);
        });
        // Create groupOutput out of groupItems
        groupOutput = groupItems.map((item) => {
            return item.getPublicValue();
        });
        // Logging
        if (this.agileInstance().config.logJobs && notFoundItemKeys.length > 0)
            console.warn(`Agile: Couldn't find some Items in Collection '${this.key}'`, notFoundItemKeys);
        this._items = groupItems.map((item) => () => item);
        this._output = groupOutput;
        this.notFoundItemKeys = notFoundItemKeys;
    }
}
exports.Group = Group;
