import { Agile, Item, Group, GroupKey, Selector, SelectorKey, StateKey, StorageKey, GroupConfigInterface, CollectionPersistent } from "../internal";
export declare class Collection<DataType = DefaultItem> {
    agileInstance: () => Agile;
    config: CollectionConfigInterface;
    size: number;
    data: {
        [key: string]: Item<DataType>;
    };
    _key?: CollectionKey;
    isPersisted: boolean;
    persistent: CollectionPersistent | undefined;
    groups: {
        [key: string]: Group<any>;
    };
    selectors: {
        [key: string]: Selector<any>;
    };
    /**
     * @public
     * Class that holds a List of Objects with key and causes rerender on subscribed Components
     * @param agileInstance - An instance of Agile
     * @param config - Config
     */
    constructor(agileInstance: Agile, config?: CollectionConfig<DataType>);
    /**
     * @public
     * Set Key/Name of Collection
     */
    set key(value: StateKey | undefined);
    /**
     * @public
     * Get Key/Name of Collection
     */
    get key(): StateKey | undefined;
    /**
     * @public
     * Set Key/Name of Collection
     * @param value - New Key/Name of Collection
     */
    setKey(value: StateKey | undefined): void;
    /**
     * @public
     * Group - Holds Items of this Collection
     * @param initialItems - Initial ItemKeys of Group
     * @param config - Config
     */
    Group(initialItems?: Array<ItemKey>, config?: GroupConfigInterface): Group<DataType>;
    /**
     * @public
     * Selector - Represents an Item of this Collection
     * @param initialKey - Key of Item that the Selector represents
     * @param config - Config
     */
    Selector(initialKey: ItemKey, config?: {
        key?: SelectorKey;
    }): Selector<DataType>;
    /**
     * @internal
     * Instantiates Groups
     */
    private initGroups;
    /**
     * @internal
     * Instantiates Selectors
     */
    private initSelectors;
    /**
     * @public
     * Collect Item/s
     * @param items - Item/s that get collected and added to this Collection
     * @param groups - Add collected Item/s to certain Groups
     * @param config - Config
     */
    collect(items: DataType | Array<DataType>, groups?: GroupKey | Array<GroupKey>, config?: CollectConfigInterface<DataType>): this;
    /**
     * @public
     * Updates Item at provided Key
     * @param itemKey - ItemKey of Item that gets updated
     * @param changes - Changes that will be merged into the Item (flatMerge)
     * @param config - Config
     */
    update(itemKey: ItemKey, changes: DefaultItem | DataType, config?: UpdateConfigInterface): Item | undefined;
    /**
     * @public
     * Creates new Group that can hold Items of Collection
     * @param groupKey - Name/Key of Group
     * @param initialItems - Initial ItemKeys of Group
     */
    createGroup(groupKey: GroupKey, initialItems?: Array<ItemKey>): Group<DataType>;
    /**
     * @public
     * Creates new Selector that represents an Item of the Collection
     * @param selectorKey - Name/Key of Selector
     * @param itemKey - Key of Item which the Selector represents
     */
    createSelector(selectorKey: SelectorKey, itemKey: ItemKey): Selector<DataType>;
    /**
     * @public
     * Get Group by Key/Name
     * @param groupKey - Name/Key of Group
     */
    getGroup(groupKey: GroupKey): Group<DataType>;
    /**
     * @public
     * Removes Group by Key/Name
     * @param groupKey - Name/Key of Selector
     */
    removeGroup(groupKey: GroupKey): this;
    /**
     * @public
     * Get Selector by Key/Name
     * @param selectorKey - Name/Key of Selector
     */
    getSelector(selectorKey: SelectorKey): Selector<DataType> | undefined;
    /**
     * @public
     * Removes Selector by Key/Name
     * @param selectorKey - Name/Key of Selector
     */
    removeSelector(selectorKey: SelectorKey): this;
    /**
     * @public
     * Remove Items from Group or from everywhere
     * @param itemKeys - ItemKey/s that get removed
     */
    remove(itemKeys: ItemKey | Array<ItemKey>): {
        fromGroups: (groups: Array<ItemKey> | ItemKey) => void;
        everywhere: () => void;
    };
    /**
     * @public
     * Get Item by Id
     * @param itemKey - ItemKey of Item that might get found
     */
    getItemById(itemKey: ItemKey): Item<DataType> | undefined;
    /**
     * @public
     * Get Value of Item by Id
     * @param itemKey - ItemKey of ItemValue that might get found
     */
    getValueById(itemKey: ItemKey): DataType | undefined;
    /**
     * @public
     * Stores Collection Value in Agile Storage
     * @param key - Storage Key (Note: not needed if Collection has key/name)
     */
    persist(key?: StorageKey): this;
    /**
     * @public
     * Get count of registered Groups in Collection
     */
    getGroupCount(): number;
    /**
     * @public
     * Get count of registered Selectors in Collection
     */
    getSelectorCount(): number;
    /**
     * @internal
     * Updates ItemKey of Item
     * @param oldItemKey - Old ItemKey
     * @param newItemKey - New ItemKey
     * @param config - Config
     */
    private updateItemKey;
    /**
     * @public
     * Removes Item/s from Group/s
     * @param itemKeys - ItemKey/s that get removed from Group/s
     * @param groupKeys - GroupKey/s of Group/s form which the ItemKey/s will be removed
     */
    removeFromGroups(itemKeys: ItemKey | Array<ItemKey>, groupKeys: GroupKey | Array<GroupKey>): void;
    /**
     * @public
     * Removes Item/s from Group/s
     * @param itemKeys - ItemKey/s of Item/s that get removed from Collection
     */
    removeItems(itemKeys: ItemKey | Array<ItemKey>): void;
    /**
     * @internal
     * Creates/Updates Item from provided Data and adds it to the Collection
     * @param data - Data from which the Item gets created/updated
     * @param config - Config
     */
    setData(data: DataType, config?: {
        patch?: boolean;
        background?: boolean;
    }): boolean;
    /**
     * @internal
     * Rebuilds Groups that include the provided ItemKey
     * @itemKey - Item Key
     * @config - Config
     */
    rebuildGroupsThatIncludeItemKey(itemKey: ItemKey, config?: RebuildGroupsThatIncludeItemKeyConfigInterface): void;
}
export declare type DefaultItem = {
    [key: string]: any;
};
export declare type CollectionKey = string | number;
export declare type ItemKey = string | number;
/**
 * @param key - Key/Name of Collection
 * @param groups - Groups of Collection
 * @param selectors - Selectors of Collection
 * @param primaryKey - Name of Property that holds the PrimaryKey (default = id)
 * @param defaultGroupKey - Key/Name of Default Group that holds all collected Items
 */
export interface CollectionConfigInterface {
    groups?: {
        [key: string]: Group<any>;
    } | string[];
    selectors?: {
        [key: string]: Selector<any>;
    } | string[];
    key?: CollectionKey;
    primaryKey?: string;
    defaultGroupKey?: ItemKey;
}
/**
 * @param patch - If Item gets patched into existing Item with the same Id
 * @param method - Way of adding Item to Collection (push, unshift)
 * @param forEachItem - Gets called for each Item that got collected
 * @param background - If collecting an Item happens in the background (-> not causing any rerender)
 */
export interface CollectConfigInterface<DataType = any> {
    patch?: boolean;
    method?: "push" | "unshift";
    forEachItem?: (data: DataType, key: ItemKey, index: number) => void;
    background?: boolean;
}
/**
 * @param addNewProperties - If properties that doesn't exist in base ItemData get added
 * @param background - If updating an Item happens in the background (-> not causing any rerender)
 */
export interface UpdateConfigInterface {
    addNewProperties?: boolean;
    background?: boolean;
}
/**
 * @param background - If updating the primaryKey of an Item happens in the background (-> not causing any rerender)
 */
export interface UpdateItemKeyConfigInterface {
    background?: boolean;
}
/**
 * @param background - If assigning a new value happens in the background (-> not causing any rerender)
 * @param forceRerender - Force rerender no matter what happens
 * @param sideEffects - If Side Effects of State get executed
 */
export interface RebuildGroupsThatIncludeItemKeyConfigInterface {
    background?: boolean;
    forceRerender?: boolean;
    sideEffects?: boolean;
}
export declare type CollectionConfig<DataType = DefaultItem> = CollectionConfigInterface | ((collection: Collection<DataType>) => CollectionConfigInterface);
