"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Collection = void 0;
const internal_1 = require("../internal");
class Collection {
    /**
     * @public
     * Class that holds a List of Objects with key and causes rerender on subscribed Components
     * @param agileInstance - An instance of Agile
     * @param config - Config
     */
    constructor(agileInstance, config = {}) {
        this.size = 0; // Amount of Items stored in Collection
        this.data = {}; // Collection Data
        this.isPersisted = false; // If Collection is stored in Storage
        this.groups = {};
        this.selectors = {};
        this.agileInstance = () => agileInstance;
        if (typeof config === "function")
            config = config(this);
        this.config = internal_1.defineConfig(config, {
            primaryKey: "id",
            groups: {},
            selectors: {},
            defaultGroupKey: "default",
        });
        this._key = this.config.key;
        this.initGroups();
        this.initSelectors();
    }
    /**
     * @public
     * Set Key/Name of Collection
     */
    set key(value) {
        this.setKey(value);
    }
    /**
     * @public
     * Get Key/Name of Collection
     */
    get key() {
        return this._key;
    }
    //=========================================================================================================
    // Set Key
    //=========================================================================================================
    /**
     * @public
     * Set Key/Name of Collection
     * @param value - New Key/Name of Collection
     */
    setKey(value) {
        const oldKey = this._key;
        // Update State Key
        this._key = value;
        // Update Key in PersistManager
        if (value !== undefined &&
            this.persistent &&
            this.persistent.key === oldKey)
            this.persistent.key = value;
    }
    //=========================================================================================================
    // Group
    //=========================================================================================================
    /**
     * @public
     * Group - Holds Items of this Collection
     * @param initialItems - Initial ItemKeys of Group
     * @param config - Config
     */
    Group(initialItems, config) {
        return new internal_1.Group(this.agileInstance(), this, initialItems, config);
    }
    //=========================================================================================================
    // Selector
    //=========================================================================================================
    /**
     * @public
     * Selector - Represents an Item of this Collection
     * @param initialKey - Key of Item that the Selector represents
     * @param config - Config
     */
    Selector(initialKey, config) {
        return new internal_1.Selector(this, initialKey, config);
    }
    //=========================================================================================================
    // Init Groups
    //=========================================================================================================
    /**
     * @internal
     * Instantiates Groups
     */
    initGroups() {
        const groups = internal_1.copy(this.config.groups);
        if (!groups)
            return;
        let groupsObject = {};
        // If groups is Array of SelectorNames transform it to Selector Object
        if (Array.isArray(groups)) {
            groups.forEach((groupKey) => {
                groupsObject[groupKey] = new internal_1.Group(this.agileInstance(), this, [], {
                    key: groupKey,
                });
            });
        }
        else
            groupsObject = groups;
        // Add default Group
        groupsObject[this.config.defaultGroupKey || "default"] = new internal_1.Group(this.agileInstance(), this, [], {
            key: this.config.defaultGroupKey || "default",
        });
        // Set Key/Name of Group to property Name
        for (let key in groupsObject)
            if (!groupsObject[key].key)
                groupsObject[key].key = key;
        this.groups = groupsObject;
    }
    //=========================================================================================================
    // Init Selectors
    //=========================================================================================================
    /**
     * @internal
     * Instantiates Selectors
     */
    initSelectors() {
        const selectors = internal_1.copy(this.config.selectors);
        if (!selectors)
            return;
        let selectorsObject = {};
        // If selectors is Array of SelectorNames transform it to Selector Object
        if (Array.isArray(selectors)) {
            selectors.forEach((selectorKey) => {
                selectorsObject[selectorKey] = new internal_1.Selector(this, selectorKey, {
                    key: selectorKey,
                });
            });
        }
        else
            selectorsObject = selectors;
        // Set Key/Name of Selector to property Name
        for (let key in selectorsObject)
            if (!selectorsObject[key].key)
                selectorsObject[key].key = key;
        this.selectors = selectorsObject;
    }
    //=========================================================================================================
    // Collect
    //=========================================================================================================
    /**
     * @public
     * Collect Item/s
     * @param items - Item/s that get collected and added to this Collection
     * @param groups - Add collected Item/s to certain Groups
     * @param config - Config
     */
    collect(items, groups, config = {}) {
        const _items = internal_1.normalizeArray(items);
        const groupKeys = internal_1.normalizeArray(groups);
        const defaultGroupKey = this.config.defaultGroupKey || "default";
        const primaryKey = this.config.primaryKey || "id";
        config = internal_1.defineConfig(config, {
            method: "push",
            background: false,
            patch: false,
        });
        // Add default GroupKey, because Items get always added to default Group
        if (!groupKeys.includes(defaultGroupKey))
            groupKeys.push(defaultGroupKey);
        // Create not existing Groups
        groupKeys.forEach((groupName) => !this.groups[groupName] && this.createGroup(groupName));
        // Instantiate Items
        _items.forEach((data, index) => {
            const itemKey = data[primaryKey];
            const itemExistsInCollection = !!this.data[itemKey];
            // Add Item to Collection
            const success = this.setData(data, {
                patch: config.patch,
                background: config.background,
            });
            if (!success)
                return this;
            // Ingest Groups that include the ItemKey into Runtime, which than rebuilds the Group (because output of group changed)
            if (!itemExistsInCollection) {
                for (let groupKey in this.groups) {
                    const group = this.getGroup(groupKey);
                    if (group.value.includes(itemKey)) {
                        if (!config.background)
                            group.ingest({ forceRerender: true, storage: false });
                    }
                }
            }
            // Add ItemKey to provided Groups
            groupKeys.forEach((groupKey) => {
                this.groups[groupKey].add(itemKey, {
                    method: config.method,
                    background: config.background,
                });
            });
            if (config.forEachItem)
                config.forEachItem(data, itemKey, index);
        });
        return this;
    }
    //=========================================================================================================
    // Update
    //=========================================================================================================
    /**
     * @public
     * Updates Item at provided Key
     * @param itemKey - ItemKey of Item that gets updated
     * @param changes - Changes that will be merged into the Item (flatMerge)
     * @param config - Config
     */
    update(itemKey, changes, config = {}) {
        if (!this.data.hasOwnProperty(itemKey)) {
            console.error(`Agile: ItemKey '${itemKey} doesn't exist in Collection!`, this);
            return undefined;
        }
        if (!internal_1.isValidObject(changes)) {
            console.error(`Agile: Changes have to be an Object!`, this);
            return undefined;
        }
        const item = this.data[itemKey];
        const primaryKey = this.config.primaryKey || "";
        config = internal_1.defineConfig(config, {
            addNewProperties: true,
            background: false,
        });
        // Merge changes into ItemValue
        const newItemValue = internal_1.flatMerge(internal_1.copy(item.nextStateValue), changes, {
            addNewProperties: config.addNewProperties,
        });
        const oldItemKey = item.value[primaryKey];
        const newItemKey = newItemValue[primaryKey];
        const updateItemKey = oldItemKey !== newItemKey;
        // Apply changes to Item
        item.set(newItemValue, {
            background: config.background,
            storage: !updateItemKey,
        });
        // Update ItemKey of Item
        if (updateItemKey)
            this.updateItemKey(oldItemKey, newItemKey, {
                background: config.background,
            });
        return this.data[newItemValue[primaryKey]];
    }
    //=========================================================================================================
    // Create Group
    //=========================================================================================================
    /**
     * @public
     * Creates new Group that can hold Items of Collection
     * @param groupKey - Name/Key of Group
     * @param initialItems - Initial ItemKeys of Group
     */
    createGroup(groupKey, initialItems) {
        if (this.groups.hasOwnProperty(groupKey)) {
            console.warn(`Agile: Group with the name '${groupKey}' already exists!`);
            return this.groups[groupKey];
        }
        // Create new Group
        const group = new internal_1.Group(this.agileInstance(), this, initialItems, { key: groupKey });
        this.groups[groupKey] = group;
        // Logging
        if (this.agileInstance().config.logJobs)
            console.log(`Agile: Created new Group called '${groupKey}'`, group);
        return group;
    }
    //=========================================================================================================
    // Create Selector
    //=========================================================================================================
    /**
     * @public
     * Creates new Selector that represents an Item of the Collection
     * @param selectorKey - Name/Key of Selector
     * @param itemKey - Key of Item which the Selector represents
     */
    createSelector(selectorKey, itemKey) {
        if (this.selectors.hasOwnProperty(selectorKey)) {
            console.warn(`Agile: The Selector with the name '${selectorKey}' already exists!`);
            return this.selectors[selectorKey];
        }
        // Create new Selector
        const selector = new internal_1.Selector(this, itemKey, {
            key: selectorKey,
        });
        this.selectors[selectorKey] = selector;
        // Logging
        if (this.agileInstance().config.logJobs)
            console.log(`Agile: Created Selector called '${selectorKey}'`, selector);
        return selector;
    }
    //=========================================================================================================
    // Get Group
    //=========================================================================================================
    /**
     * @public
     * Get Group by Key/Name
     * @param groupKey - Name/Key of Group
     */
    getGroup(groupKey) {
        if (!this.groups[groupKey]) {
            console.warn(`Agile: Group with the key/name '${groupKey}' doesn't exist!`);
            // Return empty group because it might get annoying to handle with undefined (can check if it exists with group.exists)
            const group = new internal_1.Group(this.agileInstance(), this, [], {
                key: "dummy",
            });
            group.isPlaceholder = true;
            return group;
        }
        return this.groups[groupKey];
    }
    //=========================================================================================================
    // Remove Selector
    //=========================================================================================================
    /**
     * @public
     * Removes Group by Key/Name
     * @param groupKey - Name/Key of Selector
     */
    removeGroup(groupKey) {
        if (!this.groups[groupKey]) {
            console.warn(`Agile: Group with the key/name '${groupKey}' doesn't exist!`);
            return this;
        }
        delete this.groups[groupKey];
        return this;
    }
    //=========================================================================================================
    // Get Selector
    //=========================================================================================================
    /**
     * @public
     * Get Selector by Key/Name
     * @param selectorKey - Name/Key of Selector
     */
    getSelector(selectorKey) {
        if (!this.selectors[selectorKey]) {
            console.warn(`Agile: Selector with the key/name '${selectorKey}' doesn't exist!`);
            return undefined;
        }
        return this.selectors[selectorKey];
    }
    //=========================================================================================================
    // Remove Selector
    //=========================================================================================================
    /**
     * @public
     * Removes Selector by Key/Name
     * @param selectorKey - Name/Key of Selector
     */
    removeSelector(selectorKey) {
        if (!this.selectors[selectorKey]) {
            console.warn(`Agile: Selector with the key/name '${selectorKey}' doesn't exist!`);
            return this;
        }
        delete this.selectors[selectorKey];
        return this;
    }
    //=========================================================================================================
    // Remove
    //=========================================================================================================
    /**
     * @public
     * Remove Items from Group or from everywhere
     * @param itemKeys - ItemKey/s that get removed
     */
    remove(itemKeys) {
        return {
            fromGroups: (groups) => this.removeFromGroups(itemKeys, groups),
            everywhere: () => this.removeItems(itemKeys),
        };
    }
    //=========================================================================================================
    // Get Item by Id
    //=========================================================================================================
    /**
     * @public
     * Get Item by Id
     * @param itemKey - ItemKey of Item that might get found
     */
    getItemById(itemKey) {
        if (!this.data.hasOwnProperty(itemKey) || !this.data[itemKey].exists)
            return undefined;
        const item = this.data[itemKey];
        // Add State to tracked Observers (for auto tracking used observers in computed function)
        if (this.agileInstance().runtime.trackObservers)
            this.agileInstance().runtime.foundObservers.add(item.observer);
        return item;
    }
    //=========================================================================================================
    // Get Value by Id
    //=========================================================================================================
    /**
     * @public
     * Get Value of Item by Id
     * @param itemKey - ItemKey of ItemValue that might get found
     */
    getValueById(itemKey) {
        let item = this.getItemById(itemKey);
        if (!item)
            return undefined;
        return item.value;
    }
    //=========================================================================================================
    // Persist
    //=========================================================================================================
    /**
     * @public
     * Stores Collection Value in Agile Storage
     * @param key - Storage Key (Note: not needed if Collection has key/name)
     */
    persist(key) {
        // Update Persistent Key
        if (this.persistent) {
            if (key)
                this.persistent.key = key;
            return this;
        }
        // Create persistent -> Persist Value
        this.persistent = new internal_1.CollectionPersistent(this.agileInstance(), this, key);
        return this;
    }
    //=========================================================================================================
    // Group Size
    //=========================================================================================================
    /**
     * @public
     * Get count of registered Groups in Collection
     */
    getGroupCount() {
        let size = 0;
        for (let group in this.groups)
            size++;
        return size;
    }
    //=========================================================================================================
    // Selector Size
    //=========================================================================================================
    /**
     * @public
     * Get count of registered Selectors in Collection
     */
    getSelectorCount() {
        let size = 0;
        for (let selector in this.selectors)
            size++;
        return size;
    }
    //=========================================================================================================
    // Update Item Key
    //=========================================================================================================
    /**
     * @internal
     * Updates ItemKey of Item
     * @param oldItemKey - Old ItemKey
     * @param newItemKey - New ItemKey
     * @param config - Config
     */
    updateItemKey(oldItemKey, newItemKey, config) {
        const item = this.getItemById(oldItemKey);
        config = internal_1.defineConfig(config, {
            background: false,
        });
        if (!item || oldItemKey === newItemKey)
            return;
        // Remove Item from old ItemKey and add Item to new ItemKey
        delete this.data[oldItemKey];
        this.data[newItemKey] = item;
        // Update Key/Name of Item
        item.key = newItemKey;
        // Update persist Key of Item (Doesn't get changed by setting new item key because PersistKey is not ItemKey)
        item.persist(internal_1.CollectionPersistent.getItemStorageKey(newItemKey, this.key));
        // Update Groups
        for (let groupName in this.groups) {
            const group = this.getGroup(groupName);
            if (group.isPlaceholder || !group.has(oldItemKey))
                continue;
            // Replace old ItemKey with new ItemKey
            const newGroupValue = internal_1.copy(group.value);
            newGroupValue.splice(newGroupValue.indexOf(oldItemKey), 1, newItemKey);
            group.set(newGroupValue, { background: config === null || config === void 0 ? void 0 : config.background });
        }
        // Update Selectors
        for (let selectorName in this.selectors) {
            const selector = this.getSelector(selectorName);
            if (!selector || selector.itemKey !== oldItemKey)
                continue;
            // Replace old selected ItemKey with new ItemKey
            selector.select(newItemKey, { background: config === null || config === void 0 ? void 0 : config.background });
        }
    }
    //=========================================================================================================
    // Remove From Groups
    //=========================================================================================================
    /**
     * @public
     * Removes Item/s from Group/s
     * @param itemKeys - ItemKey/s that get removed from Group/s
     * @param groupKeys - GroupKey/s of Group/s form which the ItemKey/s will be removed
     */
    removeFromGroups(itemKeys, groupKeys) {
        const _itemKeys = internal_1.normalizeArray(itemKeys);
        const _groupKeys = internal_1.normalizeArray(groupKeys);
        _itemKeys.forEach((itemKey) => {
            let removedFromGroupsCount = 0;
            // Remove ItemKey from Groups
            _groupKeys.forEach((groupKey) => {
                const group = this.getGroup(groupKey);
                if (!group)
                    return;
                group.remove(itemKey);
                removedFromGroupsCount++;
            });
            // If Item got removed from every Groups in Collection, remove it completely
            if (removedFromGroupsCount >= this.getGroupCount())
                this.removeItems(itemKey);
        });
    }
    //=========================================================================================================
    // Remove Items
    //=========================================================================================================
    /**
     * @public
     * Removes Item/s from Group/s
     * @param itemKeys - ItemKey/s of Item/s that get removed from Collection
     */
    removeItems(itemKeys) {
        const _itemKeys = internal_1.normalizeArray(itemKeys);
        _itemKeys.forEach((itemKey) => {
            var _a;
            const item = this.getItemById(itemKey);
            if (!item)
                return;
            // Remove Item from Groups
            for (let groupKey in this.groups) {
                const group = this.getGroup(groupKey);
                if (group.has(itemKey))
                    group.remove(itemKey);
            }
            // TODO
            // Remove Selectors that represents this Item
            for (let selectorKey in this.selectors) {
                const selector = this.getSelector(selectorKey);
                if ((selector === null || selector === void 0 ? void 0 : selector.itemKey) === itemKey)
                    this.removeSelector(selectorKey);
            }
            // Remove Item from Storage
            (_a = item.persistent) === null || _a === void 0 ? void 0 : _a.removeValue();
            // Remove Item from Collection
            delete this.data[itemKey];
            this.size--;
        });
    }
    //=========================================================================================================
    // Set Data
    //=========================================================================================================
    /**
     * @internal
     * Creates/Updates Item from provided Data and adds it to the Collection
     * @param data - Data from which the Item gets created/updated
     * @param config - Config
     */
    setData(data, config = {}) {
        const _data = data; // Transformed Data to any because of unknown Object (DataType)
        const primaryKey = this.config.primaryKey || "id";
        config = internal_1.defineConfig(config, {
            patch: false,
            background: false,
        });
        if (!internal_1.isValidObject(_data)) {
            console.error("Agile: Collections items has to be an object!");
            return false;
        }
        if (!_data.hasOwnProperty(primaryKey)) {
            console.error(`Agile: Collection Item needs a primary Key property called '${this.config.primaryKey}'!`);
            return false;
        }
        const itemKey = _data[primaryKey];
        let item = this.data[itemKey];
        // Create or update Item
        if (item && config.patch)
            item = item.patch(_data, { background: config.background });
        if (item && !config.patch)
            item = item.set(_data, { background: config.background });
        if (!item) {
            item = new internal_1.Item(this, _data);
            this.size++;
        }
        // Reset isPlaceholder of Item since it got an value
        if (item.isPlaceholder)
            item.isPlaceholder = false;
        // Set new Item at itemKey
        this.data[itemKey] = item;
        return true;
    }
    //=========================================================================================================
    // Rebuild Groups That Includes Item Key
    //=========================================================================================================
    /**
     * @internal
     * Rebuilds Groups that include the provided ItemKey
     * @itemKey - Item Key
     * @config - Config
     */
    rebuildGroupsThatIncludeItemKey(itemKey, config = {}) {
        config = internal_1.defineConfig(config, {
            background: false,
            forceRerender: !(config === null || config === void 0 ? void 0 : config.background),
            sideEffects: true,
        });
        // Rebuild Groups that include ItemKey
        for (let groupKey in this.groups) {
            const group = this.getGroup(groupKey);
            if (group.has(itemKey)) {
                // group.rebuild(); Not necessary because a sideEffect of the Group is to rebuild it self
                group.ingest({
                    background: config === null || config === void 0 ? void 0 : config.background,
                    forceRerender: config === null || config === void 0 ? void 0 : config.forceRerender,
                    sideEffects: config === null || config === void 0 ? void 0 : config.sideEffects,
                    storage: false,
                });
            }
        }
    }
}
exports.Collection = Collection;
