"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Selector = void 0;
const internal_1 = require("../internal");
class Selector extends internal_1.State {
    /**
     * @public
     * Represents Item of Collection
     * @param collection - Collection that contains the Item
     * @param itemKey - ItemKey of Item that the Selector represents
     * @param config - Config
     */
    constructor(collection, itemKey, config) {
        super(collection.agileInstance(), collection.getValueById(itemKey));
        this.collection = () => collection;
        this.item = undefined;
        this._itemKey = itemKey;
        this.key = config === null || config === void 0 ? void 0 : config.key;
        // Initial Select
        this.select(itemKey);
    }
    /**
     * @public
     * Set ItemKey that the Selector represents
     */
    set itemKey(value) {
        this.select(value);
    }
    /**
     * @public
     * Get ItemKey that the Selector represents
     */
    get itemKey() {
        return this._itemKey;
    }
    /**
     * @public
     * Select new ItemKey that the Selector will represents
     * @param itemKey - New ItemKey
     * @param config - Config
     */
    select(itemKey, config = {}) {
        const oldItem = this.item;
        let newItem = this.collection().getItemById(itemKey);
        config = internal_1.defineConfig(config, {
            background: false,
            sideEffects: true,
        });
        // Remove old Item from Collection if it is an Placeholder
        if (oldItem === null || oldItem === void 0 ? void 0 : oldItem.isPlaceholder)
            delete this.collection().data[this.itemKey];
        // Create dummy Item to hold reference if Item with ItemKey doesn't exist
        if (!newItem) {
            newItem = new internal_1.Item(this.collection(), { id: itemKey });
            newItem.isPlaceholder = true;
            this.collection().data[itemKey] = newItem;
        }
        // Remove Selector sideEffect from old Item
        oldItem === null || oldItem === void 0 ? void 0 : oldItem.removeSideEffect("rebuildSelector");
        this._itemKey = itemKey;
        this.item = newItem;
        // Add Selector sideEffect to Item
        newItem.addSideEffect("rebuildSelector", () => this.rebuildSelector(config));
        // Rebuild Selector for instantiating new 'selected' ItemKey properly
        this.rebuildSelector(config);
        return this;
    }
    //=========================================================================================================
    // RebuildSelector
    //=========================================================================================================
    /**
     * @public
     * Rebuilds Selector
     * @param config - Config
     */
    rebuildSelector(config = {}) {
        var _a, _b;
        config = internal_1.defineConfig(config, {
            background: false,
            sideEffects: true,
        });
        // Set Selector Value to undefined if Item doesn't exist
        if (!this.item || this.item.isPlaceholder) {
            this._value = undefined;
            return;
        }
        // Assign ItemValue to Selector
        this.nextStateValue = internal_1.copy((_a = this.item) === null || _a === void 0 ? void 0 : _a.value);
        // Fix initialStateValue and previousStateValue if they are still set from the Placeholder
        if (internal_1.equal(this.item.initialStateValue, { id: this.itemKey }))
            this.item.initialStateValue = internal_1.copy((_b = this.item) === null || _b === void 0 ? void 0 : _b.nextStateValue);
        if (internal_1.equal(this.item.previousStateValue, { id: this.itemKey }))
            this.item.previousStateValue = internal_1.copy(this.nextStateValue);
        // Ingest nextStateValue into Runtime
        this.ingest(config);
    }
}
exports.Selector = Selector;
