import { Agile } from "../internal";
import { EventObserver } from "./event.observer";
export declare class Event<PayloadType = DefaultEventPayload> {
    agileInstance: () => Agile;
    config: EventConfig;
    _key?: EventKey;
    uses: number;
    callbacks: {
        [key: string]: EventCallbackFunction<PayloadType>;
    };
    private currentTimeout;
    private queue;
    enabled: boolean;
    observer: EventObserver;
    payload: PayloadType;
    /**
     * @public
     * Event -  Class that holds a List of Functions which can be triggered at the same time
     * @param agileInstance - An instance of Agile
     * @param config - Config
     */
    constructor(agileInstance: Agile, config?: EventConfig);
    /**
     * @public
     * Set Key/Name of Event
     */
    set key(value: EventKey | undefined);
    /**
     * @public
     * Get Key/Name of Event
     */
    get key(): EventKey | undefined;
    /**
     * @public
     * Registers new Callback Function that will be called if this Event gets triggered
     * @param callback - Callback Function that gets called if the Event gets triggered
     * * @return Key of Event
     */
    on(callback: EventCallbackFunction<PayloadType>): string;
    /**
     * @public
     * Registers new Callback Function that will be called if this Event gets triggered
     * @param key - Key of Callback Function
     * @param callback - Callback Function that gets called if the Event gets triggered
     */
    on(key: string, callback: EventCallbackFunction<PayloadType>): this;
    /**
     * @public
     * Triggers Event
     * -> Calls all registered Callback Functions
     * @param payload - Payload that gets passed into the Callback Functions
     */
    trigger(payload?: PayloadType): this;
    /**
     * @public
     * Disables Event
     */
    disable(): this;
    /**
     * @public
     * Enables Event
     */
    enable(): this;
    /**
     * @public
     * Resets Event
     */
    reset(): this;
    /**
     * @public
     * Removes Callback Function at given Key
     * @param key - Key of Callback Function that gets removed
     */
    removeCallback(key: string): this;
    /**
     * @internal
     * Triggers Event
     */
    private normalTrigger;
    /**
     * @internal
     * Triggers Event with some delay (config.delay)
     */
    private delayedTrigger;
}
export declare type EventKey = string | number;
export declare type DefaultEventPayload = {
    [key: string]: any;
};
export declare type EventCallbackFunction<PayloadType = DefaultEventPayload> = (payload?: PayloadType) => void;
/**
 * @param key - Key/Name of Event
 * @param enabled - If Event can be triggered
 * @param maxUses - How often the Event can be used/triggered
 * @param delay - Delayed call of Event Callback Functions in seconds
 * @param rerender - If triggering an Event should cause a rerender
 */
export interface EventConfig {
    key?: EventKey;
    enabled?: boolean;
    maxUses?: number;
    delay?: number;
    rerender?: boolean;
}
