"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Event = void 0;
const internal_1 = require("../internal");
const event_observer_1 = require("./event.observer");
class Event {
    /**
     * @public
     * Event -  Class that holds a List of Functions which can be triggered at the same time
     * @param agileInstance - An instance of Agile
     * @param config - Config
     */
    constructor(agileInstance, config = {}) {
        this.uses = 0;
        this.callbacks = {}; // All 'subscribed' callback function
        this.queue = []; // Queue of delayed triggers
        this.enabled = true;
        this.agileInstance = () => agileInstance;
        this.config = internal_1.defineConfig(config, {
            enabled: true,
            rerender: false,
        });
        this.observer = new event_observer_1.EventObserver(agileInstance, this, [], this.config.key);
        this._key = this.config.key;
        this.enabled =
            this.config.enabled !== undefined ? this.config.enabled : true;
    }
    /**
     * @public
     * Set Key/Name of Event
     */
    set key(value) {
        this._key = value;
        this.observer.key = value;
    }
    /**
     * @public
     * Get Key/Name of Event
     */
    get key() {
        return this._key;
    }
    on(keyOrCallback, callback) {
        const generateKey = internal_1.isFunction(keyOrCallback);
        let _callback;
        let key;
        if (generateKey) {
            key = internal_1.generateId();
            _callback = keyOrCallback;
        }
        else {
            key = keyOrCallback;
            _callback = callback;
        }
        // Check if Callback is a Function
        if (!internal_1.isFunction(_callback)) {
            console.error("Agile: A Event Callback Function has to be an function!");
            return this;
        }
        // Check if Callback Function already exists
        if (this.callbacks[key]) {
            console.error(`Agile: Event Callback Function with the key/name ${key} already exists!`);
            return this;
        }
        this.callbacks[key] = _callback;
        return generateKey ? key : this;
    }
    //=========================================================================================================
    // Trigger
    //=========================================================================================================
    /**
     * @public
     * Triggers Event
     * -> Calls all registered Callback Functions
     * @param payload - Payload that gets passed into the Callback Functions
     */
    trigger(payload) {
        if (!this.enabled)
            return this;
        if (this.config.delay)
            this.delayedTrigger(payload);
        else
            this.normalTrigger(payload);
        return this;
    }
    //=========================================================================================================
    // Disable
    //=========================================================================================================
    /**
     * @public
     * Disables Event
     */
    disable() {
        this.enabled = false;
        return this;
    }
    //=========================================================================================================
    // Enable
    //=========================================================================================================
    /**
     * @public
     * Enables Event
     */
    enable() {
        this.enabled = true;
        return this;
    }
    //=========================================================================================================
    // Reset
    //=========================================================================================================
    /**
     * @public
     * Resets Event
     */
    reset() {
        this.enabled = this.config.enabled || true;
        this.uses = 0;
        if (this.currentTimeout)
            clearTimeout(this.currentTimeout);
        return this;
    }
    //=========================================================================================================
    // Remove Callback
    //=========================================================================================================
    /**
     * @public
     * Removes Callback Function at given Key
     * @param key - Key of Callback Function that gets removed
     */
    removeCallback(key) {
        delete this.callbacks[key];
        return this;
    }
    //=========================================================================================================
    // Normal Trigger
    //=========================================================================================================
    /**
     * @internal
     * Triggers Event
     */
    normalTrigger(payload) {
        // Call registered Callbacks
        for (let key in this.callbacks)
            this.callbacks[key](payload);
        // Cause rerender
        if (this.config.rerender)
            this.observer.trigger();
        this.uses++;
        // Disable Event if maxUses got reached
        if (this.config.maxUses && this.uses >= this.config.maxUses)
            this.disable();
    }
    //=========================================================================================================
    // Delayed Trigger
    //=========================================================================================================
    /**
     * @internal
     * Triggers Event with some delay (config.delay)
     */
    delayedTrigger(payload) {
        // Check if a Timeout is currently active if so add payload to queue
        if (this.currentTimeout !== undefined) {
            if (payload)
                this.queue.push(payload);
            return;
        }
        // Triggers Callback Functions and calls itself again if queue isn't empty
        const looper = (payload) => {
            this.currentTimeout = setTimeout(() => {
                this.currentTimeout = undefined;
                this.normalTrigger(payload);
                if (this.queue.length > 0)
                    looper(this.queue.shift());
            }, this.config.delay);
        };
        looper(payload);
        return;
    }
}
exports.Event = Event;
