"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Integrations = void 0;
const internal_1 = require("../internal");
class Integrations {
    /**
     * @internal
     * Integrations - Manages Integrations of Agile
     * @param agileInstance - An Instance of Agile
     */
    constructor(agileInstance) {
        this.integrations = new Set(); // All registered Integrations
        this.agileInstance = () => agileInstance;
        // Integrate initial Integrations which are static and got set external
        internal_1.Agile.initialIntegrations.forEach((integration) => this.integrate(integration));
    }
    //=========================================================================================================
    // Integrate
    //=========================================================================================================
    /**
     * @internal
     * Integrates Framework (Integration) into Agile
     * @param integration - Integration that gets registered/integrated
     */
    integrate(integration) {
        return __awaiter(this, void 0, void 0, function* () {
            // Check if integration is valid
            if (!integration.config.name) {
                console.error("Agile: Failed to integrate framework!");
                return;
            }
            // Integrate Integration/Framework
            this.integrations.add(integration);
            if (integration.config.bind)
                integration.ready = yield integration.config.bind(this.agileInstance());
            else
                integration.ready = true;
            // Logging
            if (this.agileInstance().config.logJobs)
                console.log(`Agile: Successfully integrated '${integration.config.name}'`);
        });
    }
    //=========================================================================================================
    // Update
    //=========================================================================================================
    /**
     * @internal
     * Updates Integrations
     * -> calls 'updateMethod' in registered Integrations
     * @param componentInstance - Component that gets updated
     * @param updatedData - Updated Properties with new Value (Note: properties with no value won't get passed)
     */
    update(componentInstance, updatedData) {
        this.integrations.forEach((integration) => {
            // Check if integration is ready
            if (!integration.ready) {
                console.log(`Agile: Integration '${integration.config.name}' isn't ready yet!`);
                return;
            }
            if (integration.config.updateMethod)
                integration.config.updateMethod(componentInstance, updatedData);
        });
    }
    //=========================================================================================================
    // Has Integration
    //=========================================================================================================
    /**
     * @internal
     * Checks if Agile has registered any Integration
     */
    hasIntegration() {
        return this.integrations.size > 0;
    }
}
exports.Integrations = Integrations;
