"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Runtime = void 0;
const internal_1 = require("../internal");
class Runtime {
    /**
     * @internal
     * Runtime - Performs ingested Observers
     * @param agileInstance - An instance of Agile
     */
    constructor(agileInstance) {
        // Queue system
        this.currentJob = null;
        this.jobQueue = [];
        this.notReadyJobsToRerender = []; // Jobs that are performed but not ready to rerender (wait for mount)
        this.jobsToRerender = []; // Jobs that are performed and will be rendered
        // Tracking - Used to track computed dependencies
        this.trackObservers = false; // Check if Runtime have to track Observers
        this.foundObservers = new Set(); // Observers that got tracked during the 'trackObservers' time
        this.agileInstance = () => agileInstance;
    }
    //=========================================================================================================
    // Ingest
    //=========================================================================================================
    /**
     * @internal
     * Ingests Observer into Runtime
     * -> Creates Job which will be performed by the Runtime
     * @param observer - Observer that gets performed by the Runtime
     * @param config - Config
     */
    ingest(observer, config) {
        config = internal_1.defineConfig(config, {
            perform: true,
            background: false,
            sideEffects: true,
        });
        const job = new internal_1.Job(observer, {
            background: config.background,
            sideEffects: config.sideEffects,
            storage: config.storage,
        });
        // Logging
        if (this.agileInstance().config.logJobs)
            console.log(`Agile: Created Job(${job.observer.key})`, job);
        // Add Job to JobQueue (-> no Job get missing)
        this.jobQueue.push(job);
        // Perform Job
        if (config.perform) {
            const performJob = this.jobQueue.shift();
            if (performJob)
                this.perform(performJob);
            else
                console.warn("Agile: No Job in queue!");
        }
    }
    //=========================================================================================================
    // Perform
    //=========================================================================================================
    /**
     * @internal
     * Performs Job and adds him to the rerender queue if necessary
     * @param job - Job that gets performed
     */
    perform(job) {
        this.currentJob = job;
        // Perform Job
        job.observer.perform(job);
        job.performed = true;
        if (job.rerender)
            this.jobsToRerender.push(job);
        this.currentJob = null;
        // Logging
        if (this.agileInstance().config.logJobs)
            console.log(`Agile: Completed Job(${job.observer.key})`, job);
        // Perform Jobs as long as Jobs are in queue, if no job left update/rerender Subscribers of performed Jobs
        if (this.jobQueue.length > 0) {
            const performJob = this.jobQueue.shift();
            if (performJob)
                this.perform(performJob);
            else
                console.warn("Agile: No Job in queue!");
        }
        else {
            // https://stackoverflow.com/questions/9083594/call-settimeout-without-delay
            setTimeout(() => {
                this.updateSubscribers();
            });
        }
    }
    //=========================================================================================================
    // Update Subscribers
    //=========================================================================================================
    /**
     * @internal
     * Updates/Rerenders all Subscribed Components of the Job (Observer)
     */
    updateSubscribers() {
        if (!this.agileInstance().integrations.hasIntegration()) {
            this.jobsToRerender = [];
            return;
        }
        // Subscriptions that has to be updated/rerendered (Set = For preventing double subscriptions without further checks)
        const subscriptionsToUpdate = new Set();
        // Handle Object based Jobs and check if Job is ready
        this.jobsToRerender.concat(this.notReadyJobsToRerender).forEach((job) => {
            job.observer.subs.forEach((subscriptionContainer) => {
                // Check if Subscription is ready to rerender
                if (!subscriptionContainer.ready) {
                    this.notReadyJobsToRerender.push(job);
                    if (this.agileInstance().config.logJobs)
                        console.warn("Agile: SubscriptionContainer/Component isn't ready to rerender!", subscriptionContainer);
                    return;
                }
                if (subscriptionContainer.isObjectBased)
                    this.handleObjectBasedSubscription(subscriptionContainer, job);
                subscriptionsToUpdate.add(subscriptionContainer);
            });
        });
        // Update Subscriptions that has to be updated/rerendered
        subscriptionsToUpdate.forEach((subscriptionContainer) => {
            // Call callback function if Callback based Subscription
            if (subscriptionContainer instanceof internal_1.CallbackSubscriptionContainer)
                subscriptionContainer.callback();
            // Call update method if Component based Subscription
            if (subscriptionContainer instanceof internal_1.ComponentSubscriptionContainer)
                this.agileInstance().integrations.update(subscriptionContainer.component, this.getObjectBasedProps(subscriptionContainer));
        });
        // Logging
        if (this.agileInstance().config.logJobs)
            console.log("Agile: Updated/Rerendered Subscriptions ", subscriptionsToUpdate);
        this.jobsToRerender = [];
    }
    //=========================================================================================================
    // Handle Object Based Subscription
    //=========================================================================================================
    /**
     * @internal
     * Finds updated Key of SubscriptionContainer and adds it to 'changedObjectKeys'
     * @param subscriptionContainer - Object based SubscriptionContainer
     * @param job - Job that holds the SubscriptionContainer
     */
    handleObjectBasedSubscription(subscriptionContainer, job) {
        let localKey = null;
        if (!subscriptionContainer.isObjectBased)
            return;
        // Find localKey of Job Observer in SubscriptionContainer
        for (let key in subscriptionContainer.subsObject)
            if (subscriptionContainer.subsObject[key] === job.observer)
                localKey = key;
        // Add localKey to changedObjectKeys
        if (localKey)
            subscriptionContainer.changedObjectKeys.push(localKey);
    }
    //=========================================================================================================
    // Get Object Based Props
    //=========================================================================================================
    /**
     * @internal
     * Builds Object from 'changedObjectKeys' with new Values provided by Observers
     * @param subscriptionContainer - SubscriptionContainer from which the Object gets built
     */
    getObjectBasedProps(subscriptionContainer) {
        const finalObject = {};
        // Map trough changed Keys and build finalObject
        subscriptionContainer.changedObjectKeys.forEach((changedKey) => {
            // Check if Observer at changedKey has value property, if so add it to final Object
            if (subscriptionContainer.subsObject &&
                subscriptionContainer.subsObject[changedKey]["value"])
                finalObject[changedKey] =
                    subscriptionContainer.subsObject[changedKey]["value"];
        });
        subscriptionContainer.changedObjectKeys = [];
        return finalObject;
    }
    //=========================================================================================================
    // Get Tracked Observers
    //=========================================================================================================
    /**
     * @internal
     * Returns tracked Observers and stops Runtime from tracking anymore Observers
     */
    getTrackedObservers() {
        const finalFoundObservers = this.foundObservers;
        // Reset tracking
        this.trackObservers = false;
        this.foundObservers = new Set();
        return finalFoundObservers;
    }
}
exports.Runtime = Runtime;
