import { Agile, StorageKey, StateObserver, StatePersistent, Observer, StateJobConfigInterface, StatePersistentConfigInterface } from "../internal";
export declare class State<ValueType = any> {
    agileInstance: () => Agile;
    _key?: StateKey;
    valueType?: string;
    isSet: boolean;
    isPlaceholder: boolean;
    initialStateValue: ValueType;
    _value: ValueType;
    previousStateValue: ValueType;
    nextStateValue: ValueType;
    observer: StateObserver;
    sideEffects: {
        [key: string]: (properties?: {
            [key: string]: any;
        }) => void;
    };
    isPersisted: boolean;
    persistent: StatePersistent | undefined;
    watchers: {
        [key: string]: (value: any) => void;
    };
    /**
     * @public
     * State - Class that holds one Value and causes rerender on subscribed Components
     * @param agileInstance - An instance of Agile
     * @param initialValue - Initial Value of State
     * @param key - Key/Name of State
     * @param deps - Initial deps of State
     */
    constructor(agileInstance: Agile, initialValue: ValueType, key?: StateKey, deps?: Array<Observer>);
    /**
     * @public
     * Set Value of State
     */
    set value(value: ValueType);
    /**
     * @public
     * Get Value of State
     */
    get value(): ValueType;
    /**
     * @public
     * Set Key/Name of State
     */
    set key(value: StateKey | undefined);
    /**
     * @public
     * Get Key/Name of State
     */
    get key(): StateKey | undefined;
    /**
     * @internal
     * Set Key/Name of State
     * https://github.com/microsoft/TypeScript/issues/338
     * @param value - New Key/Name of State
     */
    setKey(value: StateKey | undefined): void;
    /**
     * @public
     * Updates Value of State
     * @param value - new State Value
     * @param config - Config
     */
    set(value: ValueType, config?: SetConfigInterface): this;
    /**
     * @internal
     * Ingests nextStateValue, computedValue into Runtime
     * @param config - Config
     */
    ingest(config?: StateJobConfigInterface): this;
    /**
     * @public
     * Assign primitive type to State Value
     * Note: This function is mainly thought for JS users
     * @param type - wished Type ('String', 'Boolean', 'Array', 'Object', 'Number')
     */
    type(type: any): this;
    /**
     * @public
     * Undoes latest State Value change
     */
    undo(): void;
    /**
     * @public
     * Resets State to its initial Value
     */
    reset(): this;
    /**
     * @public
     * Patches Object with changes into State Value
     * Note: Only useful if State is an Object
     * @param targetWithChanges - Object that holds changes which get patched into State Value
     * @param config - Config
     */
    patch(targetWithChanges: object, config?: PatchConfigInterface): this;
    /**
     * @public
     * Watches State and detects State changes
     * @param callback - Callback Function that gets called if the State Value changes
     * @return Key of Watcher
     */
    watch(callback: Callback<ValueType>): string;
    /**
     * @public
     * Watches State and detects State changes
     * @param key - Key of Watcher Function
     * @param callback - Callback Function that gets called if the State Value changes
     */
    watch(key: string, callback: Callback<ValueType>): this;
    /**
     * @public
     * Removes Watcher at given Key
     * @param key - Key of Watcher that gets removed
     */
    removeWatcher(key: string): this;
    /**
     * @public
     * Checks if watcher at given Key exists
     * @param key - Key of Watcher
     */
    hasWatcher(key: string): boolean;
    /**
     * @public
     * Stores State Value into Agile Storage permanently
     * @param config - Config
     */
    persist(config?: StatePersistentConfigInterface): this;
    /**
     * @public
     * Stores State Value into Agile Storage permanently
     * @param key - Storage Key (Note: not needed if State has key/name)
     * @param config - Config
     */
    persist(key?: StorageKey, config?: StatePersistentConfigInterface): this;
    /**
     * @public
     * Creates fresh copy of State Value (-> No reference to State Value)
     */
    copy(): ValueType;
    /**
     * @public
     * Checks if State exists
     */
    get exists(): boolean;
    /**
     * @public
     * Equivalent to ===
     * @param value - Value that gets checked if its equals to the State Value
     */
    is(value: ValueType): boolean;
    /**
     * @public
     * Equivalent to !==
     * @param value - Value that gets checked if its not equals to the State Value
     */
    isNot(value: ValueType): boolean;
    /**
     * @public
     * Inverts State Value
     * Note: Only useful with boolean based States
     */
    invert(): this;
    /**
     * @internal
     * Adds SideEffect to State
     * @param key - Key of SideEffect
     * @param sideEffect - Callback Function that gets called on every State Value change
     */
    addSideEffect(key: string, sideEffect: (properties?: {
        [key: string]: any;
    }) => void): this;
    /**
     * @internal
     * Removes SideEffect at given Key
     * @param key - Key of the SideEffect that gets removed
     */
    removeSideEffect(key: string): this;
    /**
     * @internal
     * Checks if sideEffect at given Key exists
     * @param key - Key of SideEffect
     */
    hasSideEffect(key: string): boolean;
    /**
     * @internal
     * Checks if Value has correct valueType (js)
     * @param value - Value that gets checked for its correct Type
     */
    private hasCorrectType;
    /**
     * @internal
     * Returns public Value of State
     */
    getPublicValue(): ValueType;
    /**
     * @internal
     * Returns Value that gets written into the Agile Storage
     */
    getPersistableValue(): any;
}
export declare type StateKey = string | number;
/**
 * @param background - If assigning a new value happens in the background (-> not causing any rerender)
 * @param sideEffects - If Side Effects of State get executed
 * @param storage - If State value gets saved in Agile Storage (only useful if State is persisted)
 */
export interface SetConfigInterface {
    background?: boolean;
    sideEffects?: boolean;
    storage?: boolean;
}
/**
 * @param background - If assigning new value happens in the background (-> not causing any rerender)
 * @param addNewProperties - If new Properties gets added to the State Value
 */
export interface PatchConfigInterface {
    addNewProperties?: boolean;
    background?: boolean;
}
export declare type Callback<T = any> = (value: T) => void;
