"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.State = void 0;
const internal_1 = require("../internal");
class State {
    /**
     * @public
     * State - Class that holds one Value and causes rerender on subscribed Components
     * @param agileInstance - An instance of Agile
     * @param initialValue - Initial Value of State
     * @param key - Key/Name of State
     * @param deps - Initial deps of State
     */
    constructor(agileInstance, initialValue, key, deps = []) {
        this.isSet = false; // If value is not the same as initialValue
        this.isPlaceholder = false;
        this.sideEffects = {}; // SideEffects of State (will be executed in Runtime)
        this.isPersisted = false; // If State is stored in Storage
        this.watchers = {};
        this.agileInstance = () => agileInstance;
        this.initialStateValue = initialValue;
        this._key = key;
        this._value = internal_1.copy(initialValue);
        this.previousStateValue = internal_1.copy(initialValue);
        this.nextStateValue = internal_1.copy(initialValue);
        this.observer = new internal_1.StateObserver(agileInstance, this, deps, key);
    }
    /**
     * @public
     * Set Value of State
     */
    set value(value) {
        this.set(value);
    }
    /**
     * @public
     * Get Value of State
     */
    get value() {
        // Add State to tracked Observers (for auto tracking used observers in computed function)
        if (this.agileInstance().runtime.trackObservers)
            this.agileInstance().runtime.foundObservers.add(this.observer);
        return this._value;
    }
    /**
     * @public
     * Set Key/Name of State
     */
    set key(value) {
        this.setKey(value);
    }
    /**
     * @public
     * Get Key/Name of State
     */
    get key() {
        return this._key;
    }
    //=========================================================================================================
    // Set Key
    //=========================================================================================================
    /**
     * @internal
     * Set Key/Name of State
     * https://github.com/microsoft/TypeScript/issues/338
     * @param value - New Key/Name of State
     */
    setKey(value) {
        const oldKey = this._key;
        // Update State Key
        this._key = value;
        // Update Key in Observer
        this.observer.key = value;
        // Update Key in PersistManager
        if (value !== undefined &&
            this.persistent &&
            this.persistent.key === oldKey)
            this.persistent.key = value;
    }
    //=========================================================================================================
    // Set
    //=========================================================================================================
    /**
     * @public
     * Updates Value of State
     * @param value - new State Value
     * @param config - Config
     */
    set(value, config = {}) {
        config = internal_1.defineConfig(config, {
            sideEffects: true,
            background: false,
        });
        // Check value has correct Type (js)
        if (this.valueType && !this.hasCorrectType(value)) {
            console.warn(`Agile: Incorrect type (${typeof value}) was provided.`);
            return this;
        }
        // Check if value has changed
        if (internal_1.equal(this.nextStateValue, value))
            return this;
        // Ingest new value into runtime
        this.observer.ingest(value, config);
        return this;
    }
    //=========================================================================================================
    // Ingest
    //=========================================================================================================
    /**
     * @internal
     * Ingests nextStateValue, computedValue into Runtime
     * @param config - Config
     */
    ingest(config = {}) {
        config = internal_1.defineConfig(config, {
            sideEffects: true,
            background: false,
            forceRerender: false,
        });
        this.observer.ingest(internal_1.internalIngestKey, config);
        return this;
    }
    //=========================================================================================================
    // Type
    //=========================================================================================================
    /**
     * @public
     * Assign primitive type to State Value
     * Note: This function is mainly thought for JS users
     * @param type - wished Type ('String', 'Boolean', 'Array', 'Object', 'Number')
     */
    type(type) {
        const supportedTypes = ["String", "Boolean", "Array", "Object", "Number"];
        // Check if type is a supported Type
        if (!supportedTypes.includes(type.name)) {
            console.warn(`Agile: '${type}' is not supported! Supported types: String, Boolean, Array, Object, Number`);
            return this;
        }
        this.valueType = type.name.toLowerCase();
        return this;
    }
    //=========================================================================================================
    // Undo
    //=========================================================================================================
    /**
     * @public
     * Undoes latest State Value change
     */
    undo() {
        this.set(this.previousStateValue);
    }
    //=========================================================================================================
    // Reset
    //=========================================================================================================
    /**
     * @public
     * Resets State to its initial Value
     */
    reset() {
        var _a;
        this.set(this.initialStateValue);
        (_a = this.persistent) === null || _a === void 0 ? void 0 : _a.removeValue(); // Remove State from Storage (since its the initial Value)
        return this;
    }
    //=========================================================================================================
    // Patch
    //=========================================================================================================
    /**
     * @public
     * Patches Object with changes into State Value
     * Note: Only useful if State is an Object
     * @param targetWithChanges - Object that holds changes which get patched into State Value
     * @param config - Config
     */
    patch(targetWithChanges, config = {}) {
        config = internal_1.defineConfig(config, {
            addNewProperties: true,
            background: false,
        });
        if (!internal_1.isValidObject(this.nextStateValue)) {
            console.warn("Agile: You can't use the patch method on a non object States!");
            return this;
        }
        if (!internal_1.isValidObject(targetWithChanges)) {
            console.warn("Agile: TargetWithChanges has to be an object!");
            return this;
        }
        // Merge targetWithChanges into nextStateValue
        this.nextStateValue = internal_1.flatMerge(this.nextStateValue, targetWithChanges, { addNewProperties: config.addNewProperties });
        // Check if value has been changed
        if (internal_1.equal(this.value, this.nextStateValue))
            return this;
        // Ingest updated nextStateValue into Runtime
        this.ingest({ background: config.background });
        return this;
    }
    watch(keyOrCallback, callback) {
        const generateKey = internal_1.isFunction(keyOrCallback);
        let _callback;
        let key;
        if (generateKey) {
            key = internal_1.generateId();
            _callback = keyOrCallback;
        }
        else {
            key = keyOrCallback;
            _callback = callback;
        }
        // Check if Callback is a Function
        if (!internal_1.isFunction(_callback)) {
            console.error("Agile: A Watcher Callback Function has to be an function!");
            return this;
        }
        // Check if Callback Function already exists
        if (this.watchers[key]) {
            console.error(`Agile: Watcher Callback Function with the key/name ${key} already exists!`);
            return this;
        }
        this.watchers[key] = _callback;
        return generateKey ? key : this;
    }
    //=========================================================================================================
    // Remove Watcher
    //=========================================================================================================
    /**
     * @public
     * Removes Watcher at given Key
     * @param key - Key of Watcher that gets removed
     */
    removeWatcher(key) {
        delete this.watchers[key];
        return this;
    }
    //=========================================================================================================
    // Has Watcher
    //=========================================================================================================
    /**
     * @public
     * Checks if watcher at given Key exists
     * @param key - Key of Watcher
     */
    hasWatcher(key) {
        return !!this.watchers[key];
    }
    persist(keyOrConfig = {}, config = {}) {
        let _config;
        let key;
        if (internal_1.isValidObject(keyOrConfig)) {
            _config = keyOrConfig;
            key = undefined;
        }
        else {
            _config = config || {};
            key = keyOrConfig;
        }
        _config = internal_1.defineConfig(_config, {
            instantiate: true,
        });
        // Update Persistent Key
        if (this.persistent) {
            if (key)
                this.persistent.key = key;
            return this;
        }
        // Create persistent -> Persist Value
        this.persistent = new internal_1.StatePersistent(this.agileInstance(), this, key, { instantiate: _config.instantiate });
        return this;
    }
    //=========================================================================================================
    // Copy
    //=========================================================================================================
    /**
     * @public
     * Creates fresh copy of State Value (-> No reference to State Value)
     */
    copy() {
        return internal_1.copy(this.value);
    }
    //=========================================================================================================
    // Exists
    //=========================================================================================================
    /**
     * @public
     * Checks if State exists
     */
    get exists() {
        return this.getPublicValue() !== undefined && !this.isPlaceholder;
    }
    //=========================================================================================================
    // Is
    //=========================================================================================================
    /**
     * @public
     * Equivalent to ===
     * @param value - Value that gets checked if its equals to the State Value
     */
    is(value) {
        return internal_1.equal(value, this.value);
    }
    //=========================================================================================================
    // Is Not
    //=========================================================================================================
    /**
     * @public
     * Equivalent to !==
     * @param value - Value that gets checked if its not equals to the State Value
     */
    isNot(value) {
        return internal_1.notEqual(value, this.value);
    }
    //=========================================================================================================
    // Invert
    //=========================================================================================================
    /**
     * @public
     * Inverts State Value
     * Note: Only useful with boolean based States
     */
    invert() {
        if (typeof this._value !== "boolean") {
            console.warn("Agile: You can only invert boolean based States!");
            return this;
        }
        this.set(this._value);
        return this;
    }
    //=========================================================================================================
    // Add SideEffect
    //=========================================================================================================
    /**
     * @internal
     * Adds SideEffect to State
     * @param key - Key of SideEffect
     * @param sideEffect - Callback Function that gets called on every State Value change
     */
    addSideEffect(key, sideEffect) {
        if (!internal_1.isFunction(sideEffect)) {
            console.error("Agile: A sideEffect function has to be an function!");
            return this;
        }
        this.sideEffects[key] = sideEffect;
        return this;
    }
    //=========================================================================================================
    // Remove SideEffect
    //=========================================================================================================
    /**
     * @internal
     * Removes SideEffect at given Key
     * @param key - Key of the SideEffect that gets removed
     */
    removeSideEffect(key) {
        delete this.sideEffects[key];
        return this;
    }
    //=========================================================================================================
    // Has SideEffect
    //=========================================================================================================
    /**
     * @internal
     * Checks if sideEffect at given Key exists
     * @param key - Key of SideEffect
     */
    hasSideEffect(key) {
        return !!this.sideEffects[key];
    }
    //=========================================================================================================
    // Is Correct Type
    //=========================================================================================================
    /**
     * @internal
     * Checks if Value has correct valueType (js)
     * @param value - Value that gets checked for its correct Type
     */
    hasCorrectType(value) {
        let type = typeof value;
        return type === this.valueType;
    }
    //=========================================================================================================
    // Get Public Value
    //=========================================================================================================
    /**
     * @internal
     * Returns public Value of State
     */
    getPublicValue() {
        // If State Value is used internal and output represents the real state value (for instance in Group)
        if (this["output"] !== undefined)
            return this["output"];
        return this._value;
    }
    //=========================================================================================================
    // Get Persistable Value
    //=========================================================================================================
    /**
     * @internal
     * Returns Value that gets written into the Agile Storage
     */
    getPersistableValue() {
        return this.value;
    }
}
exports.State = State;
