"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StateObserver = exports.internalIngestKey = void 0;
const internal_1 = require("../internal");
exports.internalIngestKey = "THIS_IS_AN_INTERNAL_KEY_FOR_INGESTING_INTERNAL_STUFF";
class StateObserver extends internal_1.Observer {
    /**
     * @internal
     * State Observer - Handles State changes, dependencies (-> Interface to Runtime)
     * @param agileInstance - An instance of Agile
     * @param state - State
     * @param deps - Initial Dependencies of State Observer
     * @param key - Key/Name of State Observer
     */
    constructor(agileInstance, state, deps, key) {
        super(agileInstance, deps, key, state.value);
        this.state = () => state;
        this.nextStateValue = state.value;
    }
    //=========================================================================================================
    // Ingest
    //=========================================================================================================
    /**
     * @internal
     * Ingests new State Value into Runtime and applies it to the State
     * @param newStateValue - New Value of the State
     * @param config - Config
     */
    ingest(newStateValue = exports.internalIngestKey, config = {}) {
        const state = this.state();
        config = internal_1.defineConfig(config, {
            perform: true,
            background: false,
            sideEffects: true,
            forceRerender: false,
            storage: true,
        });
        // If forceRerender, set background config to false since forceRerender is 'stronger' than background
        if (config.forceRerender && config.background)
            config.background = false;
        // Grab nextState or compute State if internalIngestKey got passed
        if (newStateValue === exports.internalIngestKey) {
            if (state instanceof internal_1.Computed)
                this.nextStateValue = state.computeValue();
            else
                this.nextStateValue = state.nextStateValue;
        }
        else
            this.nextStateValue = newStateValue;
        // Check if State Value and the new Value are equals (Note: Not checking state.nextStateValue because of the internalIngestKey)
        if (internal_1.equal(state.value, this.nextStateValue) && !config.forceRerender) {
            if (this.agileInstance().config.logJobs)
                console.warn("Agile: Doesn't created job because state values are the same! ");
            return;
        }
        this.agileInstance().runtime.ingest(this, config);
    }
    //=========================================================================================================
    // Perform
    //=========================================================================================================
    /**
     * @internal
     * Performs Job from Runtime
     * @param job - Job that gets performed
     */
    perform(job) {
        var _a;
        const state = job.observer.state();
        // Set Previous State
        state.previousStateValue = internal_1.copy(state.value);
        // Set new State Value
        state._value = internal_1.copy(this.nextStateValue);
        state.nextStateValue = internal_1.copy(this.nextStateValue);
        // Store State changes in Storage
        if (job.config.storage)
            (_a = state.persistent) === null || _a === void 0 ? void 0 : _a.updateValue();
        // Set isSet
        state.isSet = internal_1.notEqual(this.nextStateValue, state.initialStateValue);
        // Reset isPlaceholder since it got an value
        if (state.isPlaceholder)
            state.isPlaceholder = false;
        // Update Observer value
        this.value = internal_1.copy(this.nextStateValue);
        // Perform SideEffects of the Perform Function
        this.sideEffects(job);
    }
    //=========================================================================================================
    // Side Effect
    //=========================================================================================================
    /**
     * @internal
     * SideEffects of Perform Function
     * @param job - Job whose SideEffects gets executed
     */
    sideEffects(job) {
        var _a;
        const state = job.observer.state();
        // Call Watchers Functions
        for (let watcherKey in state.watchers)
            if (internal_1.isFunction(state.watchers[watcherKey]))
                state.watchers[watcherKey](state.getPublicValue());
        // Call SideEffect Functions
        if ((_a = job.config) === null || _a === void 0 ? void 0 : _a.sideEffects)
            for (let sideEffectKey in state.sideEffects)
                if (internal_1.isFunction(state.sideEffects[sideEffectKey]))
                    state.sideEffects[sideEffectKey](job.config);
        // Ingest Dependencies of Observer into Runtime
        job.observer.deps.forEach((observer) => observer instanceof StateObserver &&
            observer.ingest(exports.internalIngestKey, { perform: false }));
    }
}
exports.StateObserver = StateObserver;
