"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StatePersistent = void 0;
const internal_1 = require("../internal");
class StatePersistent extends internal_1.Persistent {
    /**
     * @internal
     * State Persist Manager - Handles permanent storing of State Value
     * @param agileInstance - An instance of Agile
     * @param state - State that gets stored
     * @param key - Key of Storage property
     * @param config - Config
     */
    constructor(agileInstance, state, key, config = {}) {
        super(agileInstance);
        config = internal_1.defineConfig(config, {
            instantiate: true,
        });
        this.state = () => state;
        if (config.instantiate)
            this.instantiatePersistent(key).then((success) => {
                state.isPersisted = success;
            });
    }
    set key(value) {
        this.setKey(value);
    }
    get key() {
        return this._key;
    }
    //=========================================================================================================
    // Set Key
    //=========================================================================================================
    /**
     * @public
     * Sets Key/Name of Persistent
     * @param value - New Key/Name of Persistent
     */
    setKey(value) {
        return __awaiter(this, void 0, void 0, function* () {
            // If persistent isn't ready try to init it with the new Key
            if (!this.ready) {
                this.instantiatePersistent(value).then((success) => {
                    this.state().isPersisted = success;
                });
                return;
            }
            // Check if key has changed
            if (value === this._key)
                return;
            // Remove value with old Key
            yield this.removeValue();
            // Update Key
            this._key = value;
            // Set value with new Key
            yield this.updateValue();
        });
    }
    //=========================================================================================================
    // Load Value
    //=========================================================================================================
    /**
     * @internal
     * Loads Value from Storage
     * @return Success?
     */
    loadValue() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.ready)
                return false;
            const loadedValue = yield this.agileInstance().storage.get(this._key);
            if (loadedValue) {
                this.state().set(loadedValue, { storage: false });
                return true;
            }
            return false;
        });
    }
    //=========================================================================================================
    // Set Value
    //=========================================================================================================
    /**
     * @internal
     * Saves/Updates Value in Storage
     * @return Success?
     */
    updateValue() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.ready)
                return false;
            this.agileInstance().storage.set(this.key, this.state().getPersistableValue());
            this.state().isPersisted = true;
            return true;
        });
    }
    //=========================================================================================================
    // Remove Value
    //=========================================================================================================
    /**
     * @internal
     * Removes Value form Storage
     * @return Success?
     */
    removeValue() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.ready)
                return false;
            this.agileInstance().storage.remove(this.key);
            this.state().isPersisted = false;
            return true;
        });
    }
    //=========================================================================================================
    // Validate Key
    //=========================================================================================================
    /**
     * @internal
     * Validates Storage Key
     * @param key - Key that gets validated
     */
    validateKey(key) {
        const state = this.state();
        // Get key from State
        if (!key && state.key)
            return state.key;
        // Return null if no key found
        if (!key)
            return null;
        // Set State Key to Storage Key if State has no key
        if (!state.key)
            state.key = key;
        return key;
    }
}
exports.StatePersistent = StatePersistent;
