"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Storage = void 0;
const internal_1 = require("../internal");
class Storage {
    /**
     * @public
     * Storage - Interface for storing Items permanently
     * @param storageConfig - Config
     */
    constructor(storageConfig) {
        this.storageReady = false;
        this.storageType = "localStorage";
        this.persistentInstances = new Set();
        this.config = internal_1.defineConfig(storageConfig, {
            prefix: "agile",
            async: false,
        });
        // If Storage Methods are provided its a custom Storage
        if (storageConfig.methods)
            this.storageType = "custom";
        // Instantiate Storage
        if (this.storageType === "custom")
            this.instantiateCustomStorage();
        if (this.storageType === "localStorage")
            this.instantiateLocalStorage();
    }
    //=========================================================================================================
    // Instantiate Local Storage
    //=========================================================================================================
    /**
     * @internal
     * Instantiates Local Storage
     */
    instantiateLocalStorage() {
        // Check if Local Storage is Available
        if (!Storage.localStorageAvailable()) {
            console.warn("Agile: Local Storage is here not available, to use Storage functionalities like persist please provide a custom Storage!");
            return;
        }
        // Set StorageMethods to LocalStorageMethods
        this.config.methods = {
            get: localStorage.getItem.bind(localStorage),
            set: localStorage.setItem.bind(localStorage),
            remove: localStorage.removeItem.bind(localStorage),
        };
        this.storageReady = true;
    }
    //=========================================================================================================
    // Instantiate Custom Storage
    //=========================================================================================================
    /**
     * @internal
     * Instantiates Custom Storage
     */
    instantiateCustomStorage() {
        var _a, _b, _c, _d, _e, _f;
        // Validate Functions
        if (!internal_1.isFunction((_a = this.config.methods) === null || _a === void 0 ? void 0 : _a.get)) {
            console.error("Agile: Your GET StorageMethod isn't valid!");
            return;
        }
        if (!internal_1.isFunction((_b = this.config.methods) === null || _b === void 0 ? void 0 : _b.set)) {
            console.error("Agile: Your SET StorageMethod isn't valid!");
            return;
        }
        if (!internal_1.isFunction((_c = this.config.methods) === null || _c === void 0 ? void 0 : _c.remove)) {
            console.error("Agile: Your REMOVE StorageMethod isn't valid!");
            return;
        }
        // Check if custom Storage is async
        if (internal_1.isAsyncFunction((_d = this.config.methods) === null || _d === void 0 ? void 0 : _d.get) ||
            internal_1.isAsyncFunction((_e = this.config.methods) === null || _e === void 0 ? void 0 : _e.set) ||
            internal_1.isAsyncFunction((_f = this.config.methods) === null || _f === void 0 ? void 0 : _f.remove))
            this.config.async = true;
        this.storageReady = true;
    }
    //=========================================================================================================
    // Get
    //=========================================================================================================
    /**
     * @public
     * Gets value at provided Key
     * @param key - Key of Storage property
     */
    get(key) {
        var _a;
        if (!this.storageReady || !((_a = this.config.methods) === null || _a === void 0 ? void 0 : _a.get))
            return;
        // Async Get
        if (this.config.async)
            return this.asyncGet(key);
        // Normal Get
        const res = this.config.methods.get(this.getStorageKey(key));
        if (internal_1.isJsonString(res))
            return JSON.parse(res);
        return res;
    }
    //=========================================================================================================
    // Async Get
    //=========================================================================================================
    /**
     * @internal
     * Gets value at provided Key (async)
     * @param key - Key of Storage property
     */
    asyncGet(key) {
        return new Promise((resolve, reject) => {
            var _a;
            (_a = this.config.methods) === null || _a === void 0 ? void 0 : _a.get(this.getStorageKey(key)).then((res) => {
                if (internal_1.isJsonString(res))
                    return resolve(JSON.parse(res));
                resolve(res);
            }).catch(reject);
        });
    }
    //=========================================================================================================
    // Set
    //=========================================================================================================
    /**
     * @public
     * Saves/Updates value at provided Key
     * @param key - Key of Storage property
     * @param value - new Value that gets set
     */
    set(key, value) {
        var _a;
        if (!this.storageReady || !((_a = this.config.methods) === null || _a === void 0 ? void 0 : _a.set))
            return;
        this.config.methods.set(this.getStorageKey(key), JSON.stringify(value));
    }
    //=========================================================================================================
    // Remove
    //=========================================================================================================
    /**
     * @public
     * Removes value at provided Key
     * @param key - Key of Storage property
     */
    remove(key) {
        var _a;
        if (!this.storageReady || !((_a = this.config.methods) === null || _a === void 0 ? void 0 : _a.remove))
            return;
        this.config.methods.remove(this.getStorageKey(key));
    }
    //=========================================================================================================
    // Get Storage Key
    //=========================================================================================================
    /**
     * @internal
     * Creates Storage Key from provided key
     * @param key - Key that gets converted into a Storage Key
     */
    getStorageKey(key) {
        return `_${this.config.prefix}_${key}`;
    }
    //=========================================================================================================
    // Local Storage Available
    //=========================================================================================================
    /**
     * @internal
     * Checks if localStorage is available in this Environment
     */
    static localStorageAvailable() {
        try {
            localStorage.setItem("_myDummyKey_", "myDummyValue");
            localStorage.removeItem("_myDummyKey_");
            return true;
        }
        catch (e) {
            return false;
        }
    }
}
exports.Storage = Storage;
