"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.globalBind = exports.clone = exports.generateId = exports.notEqual = exports.equal = exports.flatMerge = exports.defineConfig = exports.isJsonString = exports.isValidUrl = exports.isAsyncFunction = exports.isFunction = exports.getAgileInstance = exports.normalizeArray = exports.isValidObject = exports.copy = void 0;
const internal_1 = require("./internal");
function copy(value) {
    if (Array.isArray(value))
        return [...value];
    if (isValidObject(value))
        return Object.assign({}, value);
    return value;
}
exports.copy = copy;
//=========================================================================================================
// Is Valid Object
//=========================================================================================================
/**
 * @internal
 * Checks if an value is a valid Object
 * https://stackoverflow.com/questions/12996871/why-does-typeof-array-with-objects-return-object-and-not-array
 * @param value - Value that is tested for its correctness
 */
function isValidObject(value) {
    function isHTMLElement(obj) {
        try {
            return obj instanceof HTMLElement;
        }
        catch (e) {
            return (typeof obj === "object" &&
                obj.nodeType === 1 &&
                typeof obj.style === "object" &&
                typeof obj.ownerDocument === "object");
        }
    }
    return (value !== null &&
        typeof value === "object" &&
        !isHTMLElement(value) &&
        !Array.isArray(value));
}
exports.isValidObject = isValidObject;
//=========================================================================================================
// Normalize Array
//=========================================================================================================
/**
 * @internal
 * Transforms Item/s to an Item Array
 * @param items - Item/s that gets transformed to an Array
 */
function normalizeArray(items) {
    if (!items)
        return [];
    return Array.isArray(items) ? items : [items];
}
exports.normalizeArray = normalizeArray;
//=========================================================================================================
// Get Instance
//=========================================================================================================
/**
 * @internal
 * Tries to get an AgileInstance from provided instance
 * If no agileInstance found it returns the global AgileInstance
 * @param instance - Instance that might hold an AgileInstance
 */
function getAgileInstance(instance) {
    try {
        // Try to find agileInstance in Instance
        if (instance instanceof internal_1.State)
            return instance.agileInstance();
        if (instance instanceof internal_1.Event)
            return instance.agileInstance();
        if (instance instanceof internal_1.Collection)
            return instance.agileInstance();
        const _instance = instance["agileInstance"];
        if (_instance)
            return instance;
        // Return global bound agileInstance (set in first instantiation of Agile)
        return globalThis.__agile__;
    }
    catch (e) {
        // fail silently
    }
    return undefined;
}
exports.getAgileInstance = getAgileInstance;
//=========================================================================================================
// Is Function
//=========================================================================================================
/**
 * @internal
 * Checks if value is a function
 * @param value - Value that gets tested if its a function
 */
function isFunction(value) {
    return typeof value === "function";
}
exports.isFunction = isFunction;
//=========================================================================================================
// Is Async Function
//=========================================================================================================
/**
 * @internal
 * Checks if value is an async function
 * @param value - Value that gets tested if its an async function
 */
function isAsyncFunction(value) {
    return isFunction(value) && value.constructor.name === "AsyncFunction";
}
exports.isAsyncFunction = isAsyncFunction;
//=========================================================================================================
// Is Valid Url
//=========================================================================================================
/**
 * @internal
 * Checks the correctness of an url
 * Resource: https://stackoverflow.com/questions/5717093/check-if-a-javascript-string-is-a-url
 * @param url - Url that gets tested for its correctness
 */
function isValidUrl(url) {
    const pattern = new RegExp("^(https?:\\/\\/)?" + // protocol
        "((([a-z\\d]([a-z\\d-]*[a-z\\d])*)\\.)+[a-z]{2,}|" + // domain name
        "((\\d{1,3}\\.){3}\\d{1,3}))" + // OR ip (v4) address
        "(\\:\\d+)?(\\/[-a-z\\d%_.~+]*)*" + // port and path
        "(\\?[;&a-z\\d%_.~+=-]*)?" + // query string
        "(\\#[-a-z\\d_]*)?$", "i");
    return pattern.test(url);
}
exports.isValidUrl = isValidUrl;
//=========================================================================================================
// Is Json String
//=========================================================================================================
/**
 * @internal
 * Checks if value is valid JsonString
 * @param value - Value that gets checked
 */
function isJsonString(value) {
    try {
        JSON.parse(value);
    }
    catch (e) {
        return false;
    }
    return true;
}
exports.isJsonString = isJsonString;
//=========================================================================================================
// Define Config
//=========================================================================================================
/**
 * @internal
 * Merges default values/properties into config object
 * @param config - Config object that receives default values
 * @param defaults - Default values object that gets merged into config object
 */
function defineConfig(config, defaults) {
    return Object.assign(Object.assign({}, defaults), config);
}
exports.defineConfig = defineConfig;
/**
 * @internal
 * Merges items into object, be aware that the merge will only happen at the top level of the object
 * @param source - Source object
 * @param changes - Changes that get merged into the source object
 * @param config - Config
 */
function flatMerge(source, changes, config = {}) {
    // Copy Source to avoid references
    const _source = copy(source);
    if (!_source)
        return _source;
    // Loop through source object an merge changes into it
    let keys = Object.keys(changes);
    keys.forEach((property) => {
        if (!config.addNewProperties && !_source[property])
            return;
        _source[property] = changes[property];
    });
    return _source;
}
exports.flatMerge = flatMerge;
//=========================================================================================================
// Equals
//=========================================================================================================
/**
 * @internal
 * Check if two values are equal
 * @param value1 - First Value
 * @param value2 - Second Value
 */
function equal(value1, value2) {
    return value1 === value2 || JSON.stringify(value1) === JSON.stringify(value2);
}
exports.equal = equal;
//=========================================================================================================
// Not Equals
//=========================================================================================================
/**
 * @internal
 * Checks if two values aren't equal
 * @param value1 - First Value
 * @param value2 - Second Value
 */
function notEqual(value1, value2) {
    return value1 !== value2 && JSON.stringify(value1) !== JSON.stringify(value2);
}
exports.notEqual = notEqual;
//=========================================================================================================
// Generate Id
//=========================================================================================================
/**
 * @internal
 * Generates random Id
 * @param length - Length of generated Id
 */
function generateId(length) {
    const characters = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
    const charactersLength = characters.length;
    let result = "";
    if (!length)
        length = 5;
    for (let i = 0; i < length; i++) {
        result += characters.charAt(Math.floor(Math.random() * charactersLength));
    }
    return result;
}
exports.generateId = generateId;
//=========================================================================================================
// Clone
//=========================================================================================================
/**
 * @internal
 * Clones a Class
 * @param instance - Instance of Class you want to clone
 */
function clone(instance) {
    const copy = Object.create(Object.getPrototypeOf(instance));
    return Object.assign(copy, instance);
}
exports.clone = clone;
//=========================================================================================================
// Global Bind
//=========================================================================================================
/**
 * @internal
 * Binds Instance Global
 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/globalThis
 * @param key - Key of Instance
 * @param instance - Instance which becomes globally accessible (globalThis.key)
 */
function globalBind(key, instance) {
    try {
        if (!globalThis[key])
            globalThis[key] = instance;
    }
    catch (e) {
        console.warn(`Agile: Failed to create global Instance called '${name}'`, instance);
    }
}
exports.globalBind = globalBind;
